#include "SeeDOpenGL.h"
#include "SeeDEngine.h"
#include "SeeDMedia.h"

#include "LogManager.h"

//

bool
SeeDEngine::Initialize()
{
    LogMessage(" -- Initialization -- ");

    if( !media.Init() ) {
        LogError("Failed to Initialize Media");
        return false;
    }
    LogMessage("...Media Initialized");

    if( !font.Initialize("bin/textures/Font.tga") ) {
        LogError("Failed to Initialize Font");
    }

    // success
    return true;
}


bool
SeeDEngine::Shutdown()
{
	LogMessage("Shutting Down the Engine...", false);

	font.Shutdown();

	LogMessage("...Success");

	// success
	return true;
}


void
Trans(const gmVector3 axis[3], const gmVector3& origin)
{
	float	naxis[16];

	memset( &naxis, 0, sizeof(float) * 16 );

	naxis[0]  = axis[0][0];
	naxis[1]  = axis[0][1];
	naxis[2]  = axis[0][2];

	naxis[4]  = axis[1][0];
	naxis[5]  = axis[1][1];
	naxis[6]  = axis[1][2];

	naxis[8]  = axis[2][0];
	naxis[9]  = axis[2][1];
	naxis[10] = axis[2][2];

	naxis[15] = 1.0f;

	glTranslatef( origin[0], origin[1], origin[2] );
	glMultMatrixf( (float*)naxis );
}


void
SeeDEngine::Draw()
{
	glMatrixMode(GL_MODELVIEW);

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	glPolygonMode(GL_FRONT, GL_FILL);
	glPolygonMode(GL_BACK, GL_FILL);

	int num = vDrawModels.size();
	for( num; --num>=0;  ) {
		glPushMatrix();

		Trans( vDrawModels[num].axis, vDrawModels[num].pos );

		media.Draw( vDrawModels[num] );

		glPopMatrix();

		vDrawModels.pop_back();
	}

	num = vPersModels.size();
	for( num; --num >= 0; ) {
		drawInfo_t *di = &vPersModels[num];

		glPushMatrix();

		Trans( di->axis, di->pos );
		media.Draw( *di );

		glPopMatrix();
	}
}


bool
SeeDEngine::RunFrame()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();

    glRotatef( -90, 1, 0, 0 );	    // put Z going up
    glRotatef(  90, 0, 0, 1 );	    // put Z going up

	//

	glColor4f(1.0f, 1.0f, 1.0f, .5f);
	glDisable(GL_TEXTURE_2D);
	glEnable(GL_LINE_SMOOTH);
	glEnable(GL_BLEND);
	glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
	glPolygonMode(GL_FRONT, GL_LINE);
	glPolygonMode(GL_BACK, GL_LINE);

	glEnable(GL_TEXTURE_2D);
	glColor4f(0.5f, 0.5f, 0.5f, 1.0f);
	glScalef(0.1f, 0.1f, 0.1f);

	ogl.BeginOrtho();

	int num = vDrawModels.size();
	for( num; --num>=0;  ) {
		drawInfo_t *di = &vDrawModels[num];

		glPushMatrix();

		Trans( di->axis, di->pos );

		media.Draw( di->index );

		glPopMatrix();

		vDrawModels.pop_back();
	}

	ogl.EndOrtho();

	return true;
}

SeeDRenderObject::SeeDRenderObject()
{
	ogl.BeginFrame();
}

SeeDRenderObject::~SeeDRenderObject()
{
//	seed.EndFrame();
}
