
#include "SeeDOpenGL.h"
#include "LogManager.h"

#include "SeeDEngine.h"
#include "SeeDFont.h"
#include "SeeDMedia.h"

//#include "mmgr.h"

#define FONT_IMG_WIDTH		256
#define FONT_IMG_HEIGHT		256
#define FONT_CHAR_WIDTH		16
#define FONT_CHAR_HEIGHT	16

#define CHAR_PER_ROW		(FONT_IMG_WIDTH / FONT_CHAR_WIDTH)

#define CHARS_PER_FONT		((FONT_IMG_WIDTH*FONT_IMG_HEIGHT) / (FONT_CHAR_WIDTH*FONT_CHAR_HEIGHT))

bool SeeDFont::Initialize( char *font_tex ) {
	texture = media.LoadTexture( font_tex );
	if( !texture ) {
		LogError("Font Initialization - Failed to load given font texture");
		return false;
	}

	this->displayList = glGenLists(CHARS_PER_FONT);
	media.GetTexture(this->texture)->Bind();

	for( int i=0; i<CHARS_PER_FONT; i++ ) {
		div_t r = div( i, CHAR_PER_ROW );
		float w = (float)FONT_CHAR_WIDTH / (float)FONT_IMG_WIDTH;
		float h = (float)FONT_CHAR_HEIGHT / (float)FONT_IMG_HEIGHT;

		float left = r.rem * w;
		float right = left + w;

		float top = r.quot * w;
		float bottom = top + h;

		glNewList(this->displayList+i, GL_COMPILE);

			glEnable(GL_BLEND);
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			glEnable(GL_TEXTURE_2D);
			glBegin(GL_TRIANGLE_STRIP);
				glTexCoord2f(right,	1.0f-bottom);	glVertex2i(16,16);
				glTexCoord2f(left,	1.0f-bottom);	glVertex2i( 0,16);
				glTexCoord2f(right,	1.0f-top);		glVertex2i(16, 0);
				glTexCoord2f(left,	1.0f-top);		glVertex2i( 0, 0);
			glEnd();

			glTranslated(FONT_CHAR_WIDTH, 0, 0);
		glEndList();
	}

	// success
	return true;
}

void SeeDFont::Shutdown() {
	glDeleteLists(this->displayList, CHARS_PER_FONT);
}

void SeeDFont::ReadyDraw() {
	media.GetTexture(this->texture)->Bind();
	glListBase(this->displayList);
}

void SeeDEngine::Draw_Text( float x, float y, char *text, ... ) {
    static char words[128];
    
    va_list arg_list;
    va_start(arg_list, text);
    vsprintf_s(words, text, arg_list);
    va_end( arg_list );
    
    glPushMatrix();
    glLoadIdentity();
    
    glTranslatef(x, y, 0);
    font.ReadyDraw();
    glCallLists(strlen(words), GL_UNSIGNED_BYTE, words);
    glPopMatrix();
}
