
#include "SeeDOpenGL.h"
#include "SeeDHeightMap.h"
#include "seed_math.h"
#include "SeeDMedia.h"
#include "SeeDFrustum.h"

#include "LogManager.h"

//

#define H_V_DIST	10.0f
#define MAX_HEIGHT	1024.0f

//

SeeDHeightMap::SeeDHeightMap()
{
    // empty
}


SeeDHeightMap::~SeeDHeightMap()
{
    this->Unload();
}


uint
SeeDHeightMap::Draw()
{
	glPushAttrib(GL_ALL_ATTRIB_BITS);

//	glEnable(GL_CULL_FACE);
//	glFrontFace(GL_CW);
//	glCullFace(GL_BACK);

//	glEnable(GL_DEPTH_TEST);
//	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

	this->viewpos = ogl.GetViewPos();

	this->DrawSky();

    this->texture->Bind();

	glEnable(GL_TEXTURE_2D);

	glLockArraysEXT(0, this->num_verts);
	glBindBufferARB(GL_ARRAY_BUFFER_ARB, this->vertex_buffer);

	glInterleavedArrays(WORLD_VERTEX_FORMAT, 0, NULL);

	const ulong num = DrawNode(&this->root, true);

	glUnbindBufferARB(GL_ARRAY_BUFFER_ARB);
	glUnbindBufferARB(GL_ELEMENT_ARRAY_BUFFER_ARB);

	glPopAttrib();

	return num;
}


bool
SeeDHeightMap::Load(char* filename)
{
	glPushAttrib(GL_ALL_ATTRIB_BITS);

	this->Unload();

	TriangleList tri_list;

	LogMessage("Map creation started from: %s", filename);

#if 0
	tex_ind = media.LoadTexture(filename);
    this->texture = media.GetTexture(tex_ind);
#else
    ImageGenerator gen;
    gen.SetImageSize(256, 256);
    this->texture = gen.CreateNoise();
#endif

	if( !BuildVertices(this->texture, tri_list ) ) {
		this->Unload();
		return false;
	}

	LogMessage("\t%i vertices created in %i triangles", this->num_verts, this->num_tris);

//	for( uint i=0; i<num_tris; ++i )
//		tris.push_back( hm_tri_t(tri_list[i]) );

	this->tris = new Triangle[ this->num_tris ];

		glLockArraysEXT(0, this->num_verts);
		SplitNode( &this->root, tri_list );
		glUnlockArraysEXT();

    LogMessage("\t%i nodes created", this->numNodes);
    LogMessage("\t%i leaves created", this->numLeaves);

//	visible_nodes.reserve(num_leaves);

	glBindBufferARB(GL_ARRAY_BUFFER_ARB, NULL);

	glPopAttrib();

	// success
	return true;
}


bool
SeeDHeightMap::BuildVertices(SeeDTexture* _texture, TriangleList& triangleList)
{
	if( !_texture )
		return false;

    this->texture = _texture;

    ImageInfo info;
    texture->GetInfo(info);

	width  = info.width;
	height = info.height;

	const size_t imageSize = width * height;
	const size_t bytespp   = info.bitspp >> 3;

	this->verts = new WorldVertex[ imageSize ];
	memset(this->verts, 0, sizeof(WorldVertex) * imageSize);

    //uint s = sizeof(WorldVertex);
    //uint t = sizeof(float) * (2 + 4 + 3 + 3);

	VectorClear(this->center);

	const float c_scale = 1.0f / 256.0f;
	const float z_scale = MAX_HEIGHT / 768.0f;	// max height is (2^8)*3 = 768

	//
	// create vertex information from image data

	byte* pixels = info.data;
	WorldVertex* pVert = this->verts;

	for( uint y=0; y<this->height; ++y )
    {
		for( uint x=0; x<this->width; ++x, ++pVert, ++this->num_verts, pixels+=bytespp )
        {
			uint h = (bytespp==4) ? pixels[3] : (pixels[0] + pixels[1] + pixels[2]) / 3;

			VectorScale(pixels, c_scale, pVert->color);
			pVert->color[3] = 1.0f;

            pVert->point.assign(H_V_DIST * x,
                                H_V_DIST * y,
                                z_scale * h);

			pVert->uv[0] = (gmType)x/(gmType)width;
			pVert->uv[1] = 1.0f - ((gmType)y/(gmType)height);

            this->center += pVert->point;
		}
	}

	//
	// assign indeces to triangles
	// calculate vertex normals

	const uint tris_per_row = (this->width-1) * 2;
	this->num_tris = (this->height-1) * tris_per_row;

	const uint n = (this->height-1) * this->width;
    uint c = 0;

	triangleList.resize(this->num_tris);

	gmVector3     normal;
	WorldVertex*  v_1;
    WorldVertex*  v_2;
    WorldVertex*  v_3;

//	for( i=0, pTri=triangleList.begin(); i<n, pTri!=triangleList.end(); ++i, ++pTri )
    TriangleList::iterator pTri = triangleList.begin();

    for( uint y=0; y<(this->height-1); ++y )
    {
        for( uint x=0; x<(this->width-1); ++x )
        {
        const uint i = (y * this->width) + x;
		// 210
		v_1 = &verts[ (pTri->tri[0] = i) ];				// 0
		v_2 = &verts[ (pTri->tri[1] = i + 1) ];			// 1
		v_3 = &verts[ (pTri->tri[2] = i + width) ];		// 2

        normal.assign(cross(v_1->point - v_2->point,
                            v_3->point - v_2->point).normalized());

        v_1->normal = (v_1->normal + normal).normalized();
        v_2->normal = (v_2->normal + normal).normalized();
        v_3->normal = (v_3->normal + normal).normalized();

		++pTri; ++c;

		// 321
		v_1 = &verts[ (pTri->tri[0] = i + width + 1) ];	// 3
		v_2 = &verts[ (pTri->tri[1] = i + width) ];		// 2
		v_3 = &verts[ (pTri->tri[2] = i + 1) ];			// 1

        normal.assign(cross(v_1->point - v_2->point,
                            v_3->point - v_2->point).normalized());

        v_1->normal = (v_1->normal + normal).normalized();
        v_2->normal = (v_2->normal + normal).normalized();
        v_3->normal = (v_3->normal + normal).normalized();

        ++pTri; ++c;
        }
	}

    TriangleList::iterator dTri = triangleList.end();
    if( pTri == dTri ) {
        int v = 1;
    }

	// success
	return this->BuildData(verts, num_verts, triangleList);
}


/*void DrawNodeBounds( hm_node_t *node ) {
	if( !node )
		return;

	vec3_t mins, maxs;
	VectorAdd( node->mins, node->center, mins );
	VectorAdd( node->maxs, node->center, maxs );

	glColor3f(1.0f, 1.0f, 1.0f);

	// draw the node's box
	glBegin(GL_LINE_STRIP);
		glVertex3f( mins[0], mins[1], mins[2] );
		glVertex3f( mins[0], maxs[1], mins[2] );
		glVertex3f( maxs[0], maxs[1], mins[2] );
		glVertex3f( maxs[0], mins[1], mins[2] );
		glVertex3f( mins[0], mins[1], mins[2] );

		glVertex3f( mins[0], mins[1], maxs[2] );
		glVertex3f( mins[0], maxs[1], maxs[2] );
		glVertex3f( maxs[0], maxs[1], maxs[2] );
		glVertex3f( maxs[0], mins[1], maxs[2] );
		glVertex3f( mins[0], mins[1], maxs[2] );
	glEnd();

	glBegin(GL_LINES);
		glVertex3f( mins[0], maxs[1], maxs[2] );
		glVertex3f( mins[0], maxs[1], mins[2] );

		glVertex3f( maxs[0], mins[1], maxs[2] );
		glVertex3f( maxs[0], mins[1], mins[2] );

		glVertex3f( maxs[0], maxs[1], maxs[2] );
		glVertex3f( maxs[0], maxs[1], mins[2] );
	glEnd();

#if 0
	// draw the node's axis
	glBegin(GL_LINES);
		glColor3f( 1.0f, 0.0f, 0.0f );
		glVertex3fv( center );
		glVertex3f( center[0] + 1.0f, center[1], center[2] );

		glColor3f( 0.0f, 1.0f, 0.0f );
		glVertex3fv( center );
		glVertex3f( center[0], center[1] + 1.0f, center[2] );

		glColor3f( 0.0f, 0.0f, 1.0f );
		glVertex3fv( center );
		glVertex3f( center[0], center[1], center[2] + 1.0f );
	glEnd();
#endif
}*/





