
#ifndef S_SEED_MD3_H
#define S_SEED_MD3_H

#include "SeeDModel.h"

#define MD3_IDENT			(('3'<<24)+('P'<<16)+('D'<<8)+'I')
#define MD3_VERSION			15

typedef float modelType;
typedef modelType modelVert[3];

typedef struct modelInfo_s {
	char  name[64];

	int   numFrames;    // number of frames
	int   numTags;      // number of tag_'s
	int   numSurfaces;  // number of surfaces

	int   numTex;       // number of textures
} modelInfo_t;

typedef struct texture_s {
	char  name[64];
	int   shaderIndex;  // for in-game use
} texture_t;

typedef struct frameInfo_s {
	modelVert  bounds[2];
	modelVert  origin;
	float      radius;
	char       name[16];
} frameInfo_t;


typedef struct tagInfo_s {
	char       name[64];
	modelVert  origin;
	modelVert  axis[3];
} tagInfo_t;


typedef struct frame_s {
	int     info;       // frame info index
	float*  verts;      // verts[num*6] -- 
} frame_t;

typedef struct baseVertex_s {
	modelVert  xyz;
	modelVert  normal;
	modelVert  color;
	TexCoord   st;
	int        index;
} baseVertex_t;

typedef struct surface_s {
	int        numVerts;
	int        numTris;
	int        numTex;

	frame_t*   frames;  // frames[frame]

	Triangle*  triangles;  // triangles[triangle*3]
	Triangle*  orderedTriangles;

	baseVertex_t*  baseVertices;

	TexCoord*   texCoords;
	texture_t*  shaders;
} surface_t;


//


class seed_md3;


class md3_surface : public SeeDModelSurface {
public:
	md3_surface()
		:	numVerts(0),			numTris(0),
			numTex(0),				numFrames(0),
			frames(NULL),			triangles(NULL),
			texCoords(NULL),		shaders(NULL),
			orderedTriangles(NULL),	baseVertices(NULL),
			vertex_buffer(0)
	{ }
	virtual ~md3_surface()
	{ this->Unload(); }

public:
	virtual bool Unload() {
		SAFE_DELETE_ARRAY(this->shaders);
		SAFE_DELETE_ARRAY(this->orderedTriangles);
		SAFE_DELETE_ARRAY(this->texCoords);

		frame_t* f = this->frames;
		for( uint frame=0; frame<numFrames; frame++, f++ ) {
			SAFE_DELETE(f->verts);
		}

		// success
		return true;
	}

	friend class SeeDMD3;

protected:
	uint        numVerts;
	uint        numTris;
	uint        numTex;
	uint        numFrames;

	frame_t*   frames;			// frames[frame]

	Triangle*  triangles;			// triangles[triangle*3]
	Triangle*  orderedTriangles;	// 

	baseVertex_t*  baseVertices;

	TexCoord*   texCoords;			// 
	texture_t*  shaders;			// 

	uint        vertex_buffer;
};


class SeeDMD3 : public SeeDModel {
public:
	SeeDMD3()
		:	numSurfaces(0),	surfaces(NULL),
			numTags(0),		tags(NULL),
			numFrames(0),	frameInfo(NULL),
			numTex(0)
	{	}

	~SeeDMD3() {
		this->Unload();
	}

	bool InitModel() {
		this->Unload();
		this->name[0] = '\0';
		this->numSurfaces = 0;
		this->numTags = 0;
		this->numFrames = 0;
		this->numTex = 0;
	}

/*
	bool Load( char *filename ) {
		if( !filename )
			return false;

		return true;
	}
*/

	bool Load( const char* filename );

	bool Load( void *buffer );

	bool Unload() {
		SAFE_DELETE_ARRAY(this->tags);
		SAFE_DELETE_ARRAY(this->frameInfo);
		SAFE_DELETE_ARRAY(this->surfaces);

		loaded = false;

		return true;
	}

	bool Draw( drawInfo_t & di );

	//---------------------------data--
protected:
//	char			name[64];

	int				numSurfaces;
	md3_surface		*surfaces;	// 1 per surface

	int				numTags;
	tagInfo_t		*tags;		// 1 per tag per frame

	int				numFrames;
	frameInfo_t		*frameInfo;	// 1 per frame

	int				numTex;


//	bool			loaded;
};

#endif
