
#ifndef S_SEED_MODEL_H
#define S_SEED_MODEL_H

#include "seed_math.h"

class ModelLoader;
class SeeDModel;

class SeeDModelSurface
{
public:
	SeeDModelSurface() { }
	virtual ~SeeDModelSurface() { }

public:
	virtual bool Unload() = 0;

	friend class ModelLoader;
	friend class SeeDModel;

protected:
};


class SeeDModel
{
public:
	SeeDModel();
	virtual ~SeeDModel();

public:
	virtual bool Init();
	virtual bool Load(const char* filename) = 0;
	virtual bool Load(void* buffer) = 0;
	virtual bool Draw(drawInfo_t& di) = 0;

	virtual bool Unload() = 0;

public:
	bool  IsLoaded() const { return loaded; }
	char* GetName() { return name; }

	friend class ModelLoader;

protected:
	char  name[64];
	bool  loaded;

	SeeDModelSurface* surfaceList;
	int   numSurfaces;
};


class SeeDNullModel : public SeeDModel
{
public:
	SeeDNullModel()
	{ loaded = true; }

public:
	bool Load( const char *filename ) { return true; }
	bool Load( void *buffer ) { return true; }
	bool Draw( drawInfo_t & di ) {
		// draw the 3 axiis
		return true;
	}
	bool Unload() { return true; }
};


class ModelLoader
{
public:
	ModelLoader();
	~ModelLoader();

	// generic loader
public:
	SeeDModel* LoadModel(const char* filename);

	// specific loaders
public:
	bool LoadMd3();

protected:
	void*      buffer;
	SeeDModel* currentData;
};



#endif
