
#include "SeeDOpenGL.h"
#include "SeeDMedia.h"
#include "SeeDFrustum.h"

#include "SeeDOctree.h"

#include <vector>
using std::vector;




uint
OctreeNode::GetPointDir(const gmVector3& point) const
{
//	bool top = true;
//	bool left = true;
//	bool front = true;

	// check to make sure it's in the bounds
//	if( point[0] > maxs[0] || point[1] > maxs[1] || point[2] > maxs[2] ||
//		point[0] < mins[0] || point[1] < mins[1] || point[2] < mins[2] )
//		return NODE_INVALID;

    const gmVector3 p(point - center);

	if( p[2] >= 0.0f ) {
		// top
		if( p[0] < 0.0f ) {
			// left
			if( p[1] < 0.0f )
				return TOP_LEFT_FRONT;
			else
				return TOP_LEFT_BACK;
		} else {
			// right
			if( p[1] < 0.0f )
				return TOP_RIGHT_FRONT;
			else
				return TOP_RIGHT_BACK;
		}
	} else {
		// bottom
		if( p[0] < 0.0f ) {
			// left
			if( p[1] < 0.0f )
				return BOTTOM_LEFT_FRONT;
			else
				return BOTTOM_LEFT_BACK;
		} else {
			// right
			if( p[1] < 0.0f )
				return BOTTOM_RIGHT_FRONT;
			else
				return BOTTOM_RIGHT_BACK;
		}
	}
}


void
SeeDOctree::SplitNode(Node* node, TriangleList& triangleList)
{
	if( !node )
		return;

	vector<TriangleList>    node_lists;
	TriangleList::iterator  pTri, dTri;
	uint  i;
	uint  num=0;

    if( node->numTriangles <= 128 ) {
        if( node->GenLeaf(triangleList, this->triangleIndex, this->tris) ) {
            ++this->numLeaves;
            ++this->numNodes;
		}

		return;
	}

	for( i=0; i<NUM_NODES; i++ ) {
		Node* new_node = (node->nodes[i] = new Node);
//		memset( new_node, NULL, sizeof(node_t) );
		node_lists.push_back(TriangleList());
	}

	// step through each triangle and determine where each one goes
	for( pTri=triangleList.begin(); pTri!=triangleList.end(); ) {
		uint d1, d2, d3, d;

		// get the location of each of the triangle's nodes
		d1 = node->GetPointDir( verts[ pTri->tri[0] ].point );
		d2 = node->GetPointDir( verts[ pTri->tri[1] ].point );
		d3 = node->GetPointDir( verts[ pTri->tri[2] ].point );

		if( d1 == NODE_INV || d2 == NODE_INV || d3 == NODE_INV ) {
			// invalid nodes
			d = d1;
		}

		d = d1;

		if( d1 != d2 || d2 != d3 ) {
			// not all in same area
//			if( d1 == d2 )
//				d = d1;
//			else
			if( d2 == d3 )
				d = d2;
//			else if( d1 == d3 )
//				d = d1;
//			else
//				d = d1;
		}

		// keep track of where this triangle was
		dTri = pTri++;
		node_lists[d].splice( node_lists[d].end(), triangleList, dTri );
        ++(node->nodes[d]->numTriangles);
		++num;
	}

    const gmVector3 mins(node->mins);
    const gmVector3 maxs(node->maxs);
    const gmVector3 center(node->center);

	for( i=0; i<NUM_NODES; i++ ) {
		Node* n = node->nodes[i];

		if( !n )
			continue;

        if( n->numTriangles == 0 ) {
			JUST_DELETE(node->nodes[i]);
			continue;
		}

#if 0
		// 3 muls + 6 assigns + 3 adds
		switch( i ) {
		case TOP_LEFT_FRONT:
            n->mins.assign(mins[0], mins[1], center[2]);
            n->maxs.assign(center[0], center[1], maxs[2]);
			break;

		case TOP_LEFT_BACK:
			n->mins.assign(mins[0], center[1], center[2] );
			n->maxs.assign(center[0], maxs[1], maxs[2] );
			break;

		case TOP_RIGHT_FRONT:
			n->mins.assign(center[0], mins[1], center[2] );
			n->maxs.assign(maxs[0], center[1], maxs[2] );
			break;

		case TOP_RIGHT_BACK:
			n->mins.assign(center[0], center[1], center[2] );
			n->maxs.assign(maxs[0], maxs[1], maxs[2] );
			break;

		case BOTTOM_LEFT_FRONT:
			n->mins.assign(mins[0], mins[1], mins[2] );
			n->maxs.assign(center[0], center[1], center[2] );
			break;

		case BOTTOM_LEFT_BACK:
			n->mins.assign(mins[0], center[1], mins[2] );
			n->maxs.assign(center[0], maxs[1], center[2] );
			break;

		case BOTTOM_RIGHT_FRONT:
			n->mins.assign(center[0], mins[1], mins[2] );
			n->maxs.assign(maxs[0], center[1], center[2] );
			break;

		case BOTTOM_RIGHT_BACK:
			n->mins.assign(center[0], center[1], mins[2] );
			n->maxs.assign(maxs[0], maxs[1], center[2] );
			break;
		};

        n->center = (n->mins + n->maxs) * 0.5f;
#else
		// 6 muls + 3 assigns + 3 adds
        n->mins = mins * 0.5f;
        n->maxs = maxs * 0.5f;

		gmVector3 c_offset;

		switch( i ) {
		case TOP_LEFT_FRONT:
			c_offset.assign(n->mins[0], n->mins[1], n->maxs[2]);
			break;

		case TOP_LEFT_BACK:
			c_offset.assign(n->mins[0], n->maxs[1], n->maxs[2]);
			break;

		case TOP_RIGHT_FRONT:
			c_offset.assign(n->maxs[0], n->mins[1], n->maxs[2]);
			break;

		case TOP_RIGHT_BACK:
			c_offset.assign(n->maxs[0], n->maxs[1], n->maxs[2]);
			break;

		case BOTTOM_LEFT_FRONT:
			c_offset.assign(n->mins[0], n->mins[1], n->mins[2]);
			break;

		case BOTTOM_LEFT_BACK:
			c_offset.assign(n->mins[0], n->maxs[1], n->mins[2]);
			break;

		case BOTTOM_RIGHT_FRONT:
			c_offset.assign(n->maxs[0], n->mins[1], n->mins[2]);
			break;

		case BOTTOM_RIGHT_BACK:
			c_offset.assign(n->maxs[0], n->maxs[1], n->mins[2]);
			break;
		};

        n->center = center + c_offset;
#endif

		SplitNode(n, node_lists[i]);
	}

    ++this->numNodes;
}


uint
SeeDOctree::DrawNode(Node* node) const
{
	uint num = 0;

	if( !node )
		return 0;

	if( !frustum.SphereInFrustum(node->center, VectorLength(node->maxs)) )
		return 0;

	if( !frustum.CubeInFrustum(node->center, node->maxs[0]) )
		return 0;

    if( node->isLeaf ) {
//		visible_nodes.push_back(node);

		num = node->Draw();

/*		if( node->num_tris > 0 ) {
			glBindBufferARB(GL_ELEMENT_ARRAY_BUFFER_ARB, node->buffer);
			glDrawElements(GL_TRIANGLES, node->num_tris*3, GL_UNSIGNED_INT, NULL);

			num = 1;
		}
*/
	} else {
		switch( node->GetPointDir(viewpos) ) {
		case TOP_LEFT_FRONT:
			num += DrawNode(node->nodes[TOP_LEFT_FRONT]);
			num += DrawNode(node->nodes[TOP_LEFT_BACK]);
			num += DrawNode(node->nodes[BOTTOM_LEFT_FRONT]);
			num += DrawNode(node->nodes[BOTTOM_LEFT_BACK]);
			num += DrawNode(node->nodes[TOP_RIGHT_FRONT]);
			num += DrawNode(node->nodes[TOP_RIGHT_BACK]);
			num += DrawNode(node->nodes[BOTTOM_RIGHT_FRONT]);
			num += DrawNode(node->nodes[BOTTOM_RIGHT_BACK]);
			break;

		case TOP_LEFT_BACK:
			num += DrawNode(node->nodes[TOP_LEFT_BACK]);
			num += DrawNode(node->nodes[TOP_LEFT_FRONT]);
			num += DrawNode(node->nodes[BOTTOM_LEFT_BACK]);
			num += DrawNode(node->nodes[BOTTOM_LEFT_FRONT]);
			num += DrawNode(node->nodes[TOP_RIGHT_BACK]);
			num += DrawNode(node->nodes[TOP_RIGHT_FRONT]);
			num += DrawNode(node->nodes[BOTTOM_RIGHT_BACK]);
			num += DrawNode(node->nodes[BOTTOM_RIGHT_FRONT]);
			break;

		case TOP_RIGHT_FRONT:
			num += DrawNode(node->nodes[TOP_RIGHT_FRONT]);
			num += DrawNode(node->nodes[TOP_RIGHT_BACK]);
			num += DrawNode(node->nodes[BOTTOM_RIGHT_FRONT]);
			num += DrawNode(node->nodes[BOTTOM_RIGHT_BACK]);
			num += DrawNode(node->nodes[TOP_LEFT_FRONT]);
			num += DrawNode(node->nodes[TOP_LEFT_BACK]);
			num += DrawNode(node->nodes[BOTTOM_LEFT_FRONT]);
			num += DrawNode(node->nodes[BOTTOM_LEFT_BACK]);
			break;

		case TOP_RIGHT_BACK:
			num += DrawNode(node->nodes[TOP_RIGHT_BACK]);
			num += DrawNode(node->nodes[TOP_RIGHT_FRONT]);
			num += DrawNode(node->nodes[BOTTOM_RIGHT_BACK]);
			num += DrawNode(node->nodes[BOTTOM_RIGHT_FRONT]);
			num += DrawNode(node->nodes[TOP_LEFT_BACK]);
			num += DrawNode(node->nodes[TOP_LEFT_FRONT]);
			num += DrawNode(node->nodes[BOTTOM_LEFT_BACK]);
			num += DrawNode(node->nodes[BOTTOM_LEFT_FRONT]);
			break;

		case BOTTOM_LEFT_FRONT:
			num += DrawNode(node->nodes[BOTTOM_LEFT_FRONT]);
			num += DrawNode(node->nodes[BOTTOM_LEFT_BACK]);
			num += DrawNode(node->nodes[TOP_LEFT_FRONT]);
			num += DrawNode(node->nodes[TOP_LEFT_BACK]);
			num += DrawNode(node->nodes[BOTTOM_RIGHT_FRONT]);
			num += DrawNode(node->nodes[BOTTOM_RIGHT_BACK]);
			num += DrawNode(node->nodes[TOP_RIGHT_FRONT]);
			num += DrawNode(node->nodes[TOP_RIGHT_BACK]);
			break;

		case BOTTOM_LEFT_BACK:
			num += DrawNode(node->nodes[BOTTOM_LEFT_BACK]);
			num += DrawNode(node->nodes[BOTTOM_LEFT_FRONT]);
			num += DrawNode(node->nodes[TOP_LEFT_BACK]);
			num += DrawNode(node->nodes[TOP_LEFT_FRONT]);
			num += DrawNode(node->nodes[BOTTOM_RIGHT_BACK]);
			num += DrawNode(node->nodes[BOTTOM_RIGHT_FRONT]);
			num += DrawNode(node->nodes[TOP_RIGHT_BACK]);
			num += DrawNode(node->nodes[TOP_RIGHT_FRONT]);
			break;

		case BOTTOM_RIGHT_FRONT:
			num += DrawNode(node->nodes[BOTTOM_RIGHT_FRONT]);
			num += DrawNode(node->nodes[BOTTOM_RIGHT_BACK]);
			num += DrawNode(node->nodes[TOP_RIGHT_FRONT]);
			num += DrawNode(node->nodes[TOP_RIGHT_BACK]);
			num += DrawNode(node->nodes[BOTTOM_LEFT_FRONT]);
			num += DrawNode(node->nodes[BOTTOM_LEFT_BACK]);
			num += DrawNode(node->nodes[TOP_LEFT_FRONT]);
			num += DrawNode(node->nodes[TOP_LEFT_BACK]);
			break;

		case BOTTOM_RIGHT_BACK:
			num += DrawNode(node->nodes[BOTTOM_RIGHT_BACK]);
			num += DrawNode(node->nodes[BOTTOM_RIGHT_FRONT]);
			num += DrawNode(node->nodes[TOP_RIGHT_BACK]);
			num += DrawNode(node->nodes[TOP_RIGHT_FRONT]);
			num += DrawNode(node->nodes[BOTTOM_LEFT_BACK]);
			num += DrawNode(node->nodes[BOTTOM_LEFT_FRONT]);
			num += DrawNode(node->nodes[TOP_LEFT_BACK]);
			num += DrawNode(node->nodes[TOP_LEFT_FRONT]);
			break;
		};
	}

	// return the number of nodes drawn
	return num;
}


