
#ifndef SEED_OGL_H_
#define SEED_OGL_H_

#include "SeeDWindow.h"
#include <GL/glew.h>

#include "seed_types.h"
#include "seed_math.h"

class SeeDOpenGL : public SeeDWindow {
public:
	SeeDOpenGL()
	 : hDC(NULL), hRC(NULL), setup_list(0), ortho_list(0)
	{ }

	bool CreateWin(WindowInfo& windowInfo, float fov);
	bool DestroyWin();

	void Resize(int width, int height);
//	bool ChangeResolution( int width, int height, int bpp );

	bool SetFOV(float fov) { render_data.fov_y = fov; }

	bool ToggleFullscreen();

	void SetView(const gmVector3& pos, const gmVector3& angles);

	bool CheckExtension(char* ext);

	// Rendertime Functions
public:
	void BeginFrame();
	void EndFrame();

	void BeginOrtho();
	void EndOrtho();

public:
	HDC   GetDeviceContext() const;
	HGLRC GetRenderingContext() const;

	const gmVector3 GetViewPos() const;
	float GetFOV() const;
	void  GetAxis(gmVector3 axis[3]) const;

	const gmVector4 CalcFrusSphere(float near, float far) const;

	void  GetInverseModelView(gmType* mat);

protected:
	bool Initialize();   // Initializes based on render_data
	bool InitExtensions();

protected:
	// fullscreen info
	struct render_data_s {
//		uint	mode;
//		uint	x, y;
//		uint	width, height;
//		BYTE	bpp;
//		bool	fullscreen;

		float      fov_y;
		gmVector3  v_angles;
		gmVector3  v_axis[3];
		gmVector3  v_origin;
	} render_data;

	struct config_s {
		const byte	*vendor_string;
		const byte	*renderer_string;
		const byte	*version_string;
		const byte	*extensions_string;

		uint	version;
	} config;

protected:
	uint	setup_list;
	uint	ortho_list;

	// Handles
protected:
	HDC		hDC;		// Handle to the window's graphics device context
	HGLRC	hRC;		// Handle to the window's rendering context
};

//

inline HDC
SeeDOpenGL::GetDeviceContext() const
{
    return this->hDC;
}

inline HGLRC
SeeDOpenGL::GetRenderingContext() const
{
    return this->hRC;
}

inline const gmVector3
SeeDOpenGL::GetViewPos() const
{
	return this->render_data.v_origin;
}

inline float
SeeDOpenGL::GetFOV() const
{
    return this->render_data.fov_y;
}

inline void
SeeDOpenGL::GetAxis(gmVector3 axis[3]) const
{
	AxisCopy(this->render_data.v_axis, axis);
}

inline void
glVertex3gm(const gmVector3& vert)
{
    glVertex3f(vert[0], vert[1], vert[2]);
}


//

extern SeeDOpenGL ogl;

#define glUnbindBufferARB(mode)  glBindBufferARB((mode), 0)

#define VP_GPA(t, s)  ((s)=(t)wglGetProcAddress(#s))

void Trans(const gmVector3 axis[3], const gmVector3& origin);


#endif
