
#ifndef S_SEED_ORGANIZED_SPACE_H
#define S_SEED_ORGANIZED_SPACE_H

#include "seed_math.h"
#include "SeeDOpenGL.h"
#include "SeeDSpace.h"

//

template<int size>
class SpatialNode {
public:
	SpatialNode()
	 : triangles(NULL), numTriangles(0), buffer(0), isLeaf(false)
	{ for(int i=0;i<size;++i) nodes[i]=NULL; }

	virtual ~SpatialNode()
	{ Unload(); }

public:
	void Unload() {
		if( this->isLeaf )
			glDeleteBuffersARB(1, &this->buffer);
		else
        {
			for( uint i=0; i<size; ++i ) {
				if( this->nodes[i] ) {
					this->nodes[i]->Unload();
					JUST_DELETE(this->nodes[i]);
				}
			}
		}
	}

	virtual uint Divide(TriangleList& triangleList) { return 0; }
	virtual byte GetPointDir(const gmVector3& point) { return 0; }

	bool GenLeaf(TriangleList& triangleList, uint& triangleIndex, Triangle* baseTriangles) {
		TriangleList::iterator  pTri;
		uint num = 0;

		this->isLeaf = true;

		this->triangles = &baseTriangles[ triangleIndex ]; // where the hell did this come from?

		for( pTri=triangleList.begin(); pTri!=triangleList.end(); ++pTri, ++triangleIndex, ++num )
			VectorCopy(pTri->tri, baseTriangles[ triangleIndex ]);

		if( num != numTriangles ) {
			num = numTriangles;
			return false;
		}

		glGenBuffersARB(1, &this->buffer);
		glBindBufferARB(GL_ELEMENT_ARRAY_BUFFER_ARB, this->buffer);
        glBufferDataARB(GL_ELEMENT_ARRAY_BUFFER_ARB, numTriangles*sizeof(Triangle),
                        this->triangles, GL_STATIC_DRAW_ARB);
		glUnbindBufferARB(GL_ELEMENT_ARRAY_BUFFER_ARB);

		return true;
	}

	uint Draw() {
        if( this->isLeaf && this->numTriangles > 0 ) {
            glBindBufferARB(GL_ELEMENT_ARRAY_BUFFER_ARB, this->buffer);
            glDrawElements(GL_TRIANGLES, this->numTriangles*3, GL_UNSIGNED_INT, NULL);

			return 1;
		}

		return 0;
	}

	void CalcBox() {
        box[0].assign(mins[0], maxs[1], maxs[2]);
		box[1].assign(maxs[0], maxs[1], maxs[2]);
		box[2].assign(mins[0], maxs[1], mins[2]);
		box[3].assign(maxs[0], maxs[1], mins[2]);
		box[4].assign(mins[0], mins[1], maxs[2]);
		box[5].assign(maxs[0], mins[1], maxs[2]);
		box[6].assign(mins[0], mins[1], mins[2]);
		box[7].assign(maxs[0], mins[1], mins[2]);
		for(int i=0;i<8;++i)
            box[i] += center;
	}

	void DrawBox() {
		glColor3f(0.0f, 1.0f, 0.0f);

		glBegin(GL_LINE_STRIP);
			glVertex3gm( box[0] );
			glVertex3gm( box[1] );
			glVertex3gm( box[3] );
			glVertex3gm( box[2] );
			glVertex3gm( box[0] );

			glVertex3gm( box[4] );
			glVertex3gm( box[5] );
			glVertex3gm( box[7] );
			glVertex3gm( box[6] );
			glVertex3gm( box[4] );
		glEnd();

		glBegin(GL_LINES);
			glVertex3gm( box[2] );
			glVertex3gm( box[6] );

			glVertex3gm( box[1] );
			glVertex3gm( box[5] );

			glVertex3gm( box[3] );
			glVertex3gm( box[7] );

			glColor3f( 1.0f, 0.0f, 0.0f );
			glVertex3gm( center );
			glVertex3gm( box[1] );

			glColor3f( 0.0f, 0.0f, 1.0f );
			glVertex3gm( center );
			glVertex3gm( box[6] );
		glEnd();
	}

public:
	gmVector3  center;
	gmVector3  mins, maxs;
	gmVector3  box[8];

	Triangle*  triangles;
	uint       numTriangles;

	uint  buffer;			// vertex buffer
	bool  isLeaf;

	SpatialNode*  nodes[size];
};

//

template< int size, class Node_=SpatialNode<size> >
class SeeDOrganizedSpace : public SeeDSpace
{
public:
	SeeDOrganizedSpace()
	 : numNodes(0), numLeaves(0), triangleIndex(0)
	{ }

	virtual ~SeeDOrganizedSpace()
	{ Unload(); }

    typedef Node_ Node;

public:
	void Unload() {
		SeeDSpace::Unload();

        this->triangleIndex = 0;
		this->root.Unload();
	}

	uint Draw() {
		glPushAttrib(GL_ALL_ATTRIB_BITS);

//		glEnable(GL_CULL_FACE);
//		glFrontFace(GL_CW);
//		glCullFace(GL_BACK);

//		glEnable(GL_DEPTH_TEST);
//		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

		this->viewpos = ogl.GetViewPos();

		this->DrawSky();

		glDisable(GL_TEXTURE_2D);

        glLockArraysEXT(0, this->num_verts);
		glBindBufferARB(GL_ARRAY_BUFFER_ARB, this->vertex_buffer);

		glInterleavedArrays(WORLD_VERTEX_FORMAT, 0, NULL);
		ulong num = this->DrawNode(&this->root, true);

		glUnbindBufferARB(GL_ARRAY_BUFFER_ARB);
		glUnbindBufferARB(GL_ELEMENT_ARRAY_BUFFER_ARB);

		glPopAttrib();

		return num;
	}


protected:

	// 
protected:
	bool BuildData(WorldVertex* vertices, uint numVerts, TriangleList& triangleList) {
		if( !SeeDSpace::BuildData(vertices, numVerts, triangleList) )
			return false;

        root.center = this->center;
        root.mins = this->mins;
        root.maxs = this->maxs;
        root.numTriangles = this->num_tris;

		return true;
	}

	//
	// Node Stuff
protected:
	virtual void SplitNode(Node* node, TriangleList& triangleList) = 0;
	virtual uint DrawNode(Node* node, bool checkChildren=true) const = 0;

protected:
	uint    triangleIndex;

	uint    numNodes;
	uint    numLeaves;

	Node    root;

protected:
	gmVector3  viewpos;
//	node_v     visible_nodes;	// 
//	node_v     draw_nodes;		// nodes that -will- be drawn
};


#endif
