
#include "SeeDOpenGL.h"
#include "SeeDMedia.h"
#include "SeeDSpace.h"


void
AddToBounds(const gmVector3& point, gmVector3& _min, gmVector3& _max)
{
	if( point[0] > _max[0] )
		_max[0] = point[0];
	if( point[0] < _min[0] )
		_min[0] = point[0];

	if( point[1] > _max[1] )
		_max[1] = point[1];
	if( point[1] < _min[1] )
		_min[1] = point[1];

	if( point[2] > _max[2] )
		_max[2] = point[2];
	if( point[2] < _min[2] )
		_min[2] = point[2];
}


SeeDSpace::SeeDSpace()
 : verts(NULL), tris(NULL), num_verts(0),
   num_tris(0), vertex_buffer(0)
{
    // empty
}


void
SeeDSpace::Unload()
{
	SAFE_DELETE_ARRAY(verts);
	SAFE_DELETE_ARRAY(tris);

	num_verts = 0;
	num_tris = 0;

	glDeleteBuffersARB(1, &vertex_buffer);
}


bool
SeeDSpace::BuildData(WorldVertex* vertices, uint numVerts, TriangleList& tri_list)
{
	if( !vertices )
		return false;

	WorldVertex* pVert;

	//
	// center the map

	VectorClear(mins);
	VectorClear(maxs);

	float t = 1.0f / (float)numVerts;
	// the center is the average of all the points
	VectorScale(center, t, center);
	center[2] = 0;

	long j;

	// move all the points to where the center is at 0,0,0
	for( j=numVerts, pVert=vertices; --j>=0; ++pVert ) {
		VectorSubtract(pVert->point, center, pVert->point);
		AddToBounds(pVert->point, mins, maxs);
	}

//	VectorAdd(mins, maxs, center);
//	VectorScale(center, 0.5f, center);
	VectorClear(center);

	float width = fabs(maxs[0]);
	if( fabs(maxs[1]) > width )
		width = fabs(maxs[1]);
	if( fabs(maxs[2]) > width )
		width = fabs(maxs[2]);

	if( fabs(mins[0]) > width )
		width = fabs(mins[0]);
	if( fabs(mins[1]) > width )
		width = fabs(mins[1]);
	if( fabs(mins[2]) > width )
		width = fabs(mins[2]);

//	VectorSet(maxs, width, width, width );
	width = 0.0f - width;
//	VectorSet(mins, width, width, width );

//	VectorCopy(mins, root.mins);
//	VectorCopy(maxs, root.maxs);
//	VectorCopy(center, root.center);

	glGenBuffersARB(1, &vertex_buffer);
	glBindBufferARB(GL_ARRAY_BUFFER_ARB, vertex_buffer);
	glBufferDataARB(GL_ARRAY_BUFFER_ARB, num_verts*sizeof(WorldVertex), vertices, GL_STATIC_DRAW_ARB);
	glUnbindBufferARB(GL_ARRAY_BUFFER_ARB);

	ConstructSky();

	// success
	return true;
}

gmVector3 colors[] = {
	gmVector3(1.0f, 0.0f, 0.0f),
	gmVector3(0.0f, 1.0f, 0.0f),
	gmVector3(0.0f, 0.0f, 1.0f),
	gmVector3(1.0f, 1.0f, 0.0f),
	gmVector3(0.0f, 1.0f, 1.0f),
	gmVector3(1.0f, 0.0f, 1.0f),
	gmVector3(1.0f, 1.0f, 1.0f),
	gmVector3(1.0f, 1.0f, 1.0f),
};

bool
SeeDSpace::ConstructSky()
{
	// square it out
	gmType m = maxs[0];
	if( maxs[1] > m )
		m = maxs[1];
	if( maxs[2] > m )
		m = maxs[2];

	gmVector3 _min(-m, -m, -m);
    gmVector3 _max(m, m, m);

	skyBox.sides[0].texture = media.LoadTexture("textures/sky/sky_ft.tga");
	skyBox.sides[1].texture = media.LoadTexture("textures/sky/sky_lf.tga");
	skyBox.sides[2].texture = media.LoadTexture("textures/sky/sky_bk.tga");
	skyBox.sides[3].texture = media.LoadTexture("textures/sky/sky_rt.tga");
	skyBox.sides[4].texture = media.LoadTexture("textures/sky/sky_up.tga");
	skyBox.sides[5].texture = media.LoadTexture("textures/sky/sky_dn.tga");

    skyBox.points[0].point.assign(_min[0], _max[1], _max[2]);
	skyBox.points[1].point.assign(_max[0], _max[1], _max[2]);
	skyBox.points[2].point.assign(_min[0], _max[1], _min[2]);
	skyBox.points[3].point.assign(_max[0], _max[1], _min[2]);
	skyBox.points[4].point.assign(_min[0], _min[1], _max[2]);
	skyBox.points[5].point.assign(_max[0], _min[1], _max[2]);
	skyBox.points[6].point.assign(_min[0], _min[1], _min[2]);
	skyBox.points[7].point.assign(_max[0], _min[1], _min[2]);

	VectorSet4(skyBox.sides[0].quad, 0, 2, 3, 1);
	VectorSet4(skyBox.sides[1].quad, 1, 3, 7, 5);
	VectorSet4(skyBox.sides[2].quad, 5, 7, 6, 4);
	VectorSet4(skyBox.sides[3].quad, 4, 6, 2, 0);
	VectorSet4(skyBox.sides[4].quad, 5, 4, 0, 1);
	VectorSet4(skyBox.sides[5].quad, 2, 6, 7, 3);

	WorldVertex* points = skyBox.points;
	uint i;

	for( i=0; i<6; ++i ) {
		SkyBox::box_side_s* side = &skyBox.sides[i];
		uint*  quad	= side->quad;

		const gmVector3 p1(points[ quad[0] ].point);
		const gmVector3 p2(points[ quad[1] ].point);
		const gmVector3 p3(points[ quad[2] ].point);

        const gmVector3 normal(cross(p3 - p2, p1 - p2).normalized());

        points[ quad[0] ].normal += normal;
        points[ quad[1] ].normal += normal;
        points[ quad[2] ].normal += normal;
        points[ quad[3] ].normal += normal;
	}

	for(i=0; i<8; ++i) {
        points[i].normal.normalize();
//		points[i].color.assign(1.0 / (i+1), 1.0 - ((i%8)/8), 1.0 - ((8-i)/8));
        points[i].color.assign(1.0f, 1.0f, 1.0f, 1.0f);
	}

	return true;
}



void SeeDSpace::DrawSky()
{
	glPushAttrib(GL_ALL_ATTRIB_BITS);
//	glEnable(GL_CULL_FACE);

	glUnbindBufferARB(GL_ELEMENT_ARRAY_BUFFER_ARB);
	glUnbindBufferARB(GL_ARRAY_BUFFER_ARB);

	glInterleavedArrays(WORLD_VERTEX_FORMAT, 0, this->skyBox.points);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	glEnable(GL_TEXTURE_2D);

//	glTexCoordPointer(2, GL_FLOAT, sizeof(vec_t)*2, 
	for( uint i=0; i<6; ++i )
    {
        const SkyBox::box_side_s* side = &this->skyBox.sides[i];

		media.GetTexture( side->texture )->Bind();

		glBegin(GL_QUADS);
			glTexCoord2f(0.0f, 1.0f);
			glArrayElement(side->quad[0]);

			glTexCoord2f(0.0f, 0.0f);
			glArrayElement(side->quad[1]);

			glTexCoord2f(1.0f, 0.0f);
			glArrayElement(side->quad[2]);

			glTexCoord2f(1.0f, 1.0f);
			glArrayElement(side->quad[3]);
		glEnd();
	}
	glDisable(GL_TEXTURE_2D);

	glPopAttrib();
}

