#ifndef S_SEED_SPACE_H
#define S_SEED_SPACE_H

#include <list>
using std::list;

#include "seed_types.h"

// glInterleavedArrays GL_T2F_C4F_N3F_V3F compatible -- >> GL_C4F_N3F_V3F
typedef struct WorldVertex_s
{
	TexCoord   uv;
	gmVector4  color;
	gmVector3  normal;
	gmVector3  point;
} WorldVertex;

#define WORLD_VERTEX_FORMAT GL_T2F_C4F_N3F_V3F

class CTriangle {
public:
	CTriangle()            { tri[0]=tri[1]=tri[2]=0; }
	CTriangle(Triangle t)  { tri[0]=t[0]; tri[1]=t[1]; tri[2]=t[2]; }

	Triangle tri;
};


typedef list<CTriangle>  TriangleList;


class SeeDSpace {
public:
	SeeDSpace();

public:
	virtual void Unload();
	virtual uint Draw() = 0;

protected:
	virtual bool BuildData(WorldVertex* vertices, uint numVerts, TriangleList& tri_list);

protected:
	struct SkyBox {
		WorldVertex points[8];

		struct box_side_s {
			quad_t       quad;
			gmVector4    normal;

			textureIndex texture;
		} sides[6];
	} skyBox;

	bool ConstructSky();
	void DrawSky();

protected:
	uint  vertex_buffer;

	WorldVertex*  verts;
	Triangle*     tris;		// complete?

	uint  num_verts;
	uint  num_tris;

	gmVector3  center;
	gmVector3  maxs, mins;

};


void AddToBounds(const gmVector3& point, gmVector3& _min, gmVector3& _max);


#endif
