
#ifndef S_SEED_TEXTURE_H
#define S_SEED_TEXTURE_H

#include <stdio.h>
#include <memory.h>

#include <string>

using std::string;

#include "seed_types.h"

typedef struct ImageInfo_s {
	uint   width;
	uint   height;
	uint   bitspp;
	byte*  data;
    uint   index;
} ImageInfo;

class ImageLoader;
class ImageGenerator;

class SeeDTexture
{
private:
	SeeDTexture();

protected:
public:
    SeeDTexture(uint  _textureType, uint _internalFormat,
                uint _width, uint _height, uint _dataFormat,
                byte* _data);
	~SeeDTexture();

    bool Create();
	bool Destroy();

public:
	void  Bind() const;
    void  GetInfo(ImageInfo& info) const;

	const char* GetName() const { return this->name; }
    uint  GetIndex() const { return this->index; }

	friend class ImageLoader;
    friend class ImageGenerator;

protected:
	char   name[64];

	uint   textureType;
	uint   dataFormat;
	uint   width;
	uint   height;
	uint   internalFormat;
	byte*  data;

    uint   bitspp;
	uint   index;  // opengl drawing index
};

//

class ImageLoader
{
public:
	ImageLoader();
	ImageLoader(const string& filename);
	~ImageLoader();

	// generic loader
public:
	SeeDTexture* Load(const string& filename) const;
    bool         Unload(SeeDTexture* texture) const;
};

class ImageGenerator
{
public:
    ImageGenerator();

public:
    void SetImageSize(uint width, uint height);

    SeeDTexture* CreateNoise() const;
    SeeDTexture* CreateStripe(const gmVector3& color1,
                              const gmVector3& color2,
                              gmType stripeWidth);
    SeeDTexture* CreateChecker(const gmVector3& color1,
                               const gmVector3& color2,
                               gmType stripeWidth);

protected:
    uint   imageWidth;
    uint   imageHeight;
};


//


typedef struct ImageLoadingInfo_s {
	byte*  data;
	uint   bitspp;
	uint   width;
	uint   height;

	uint   dataFormat;
	uint   internalFormat;
} ImageLoadingInfo;


bool LoadTarga(const string& filename, ImageLoadingInfo& info);
bool LoadJPEG(const string& filename, ImageLoadingInfo& info);




#endif
