
#include "SeeDEngine.h"
#include "SeeDMedia.h"
#include "SeeDHeightMap.h"
#include "SeeDFrustum.h"

#include "SeeDOpenGL.h"
#include "LogManager.h"

#include <gmVec3.h>

//#include "mmgr.h"

//mdxm_model		model;
//mdxm_animations	animations;

SeeDEngine  seed;
SeeDOpenGL  ogl;

bool active = true;
bool minimized = false;
bool keys[256];


void AppActivate( bool bAct, bool bMin ) {
	minimized = bMin;

	if( bAct && !bMin )
		active = true;
	else
		active = false;

	if( !active ) {
		// deactivate everything
	} else {
		// activate everything
	}
}

LRESULT CALLBACK WindowProc( HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam ) {
	switch( msg ) {
	case WM_ACTIVATE:
		{
/*			int fAct, fMin;
			fAct = LOWORD(wparam);
			fMin = (BOOL)HIWORD(wparam);

			AppActivate( fAct != WA_INACTIVE, fMin );
*/
			if( !HIWORD(wparam) ) {
				active = true;
			} else {
				active = false;
			}
		}
		return false;

//	case WM_DESTROY:		//If the user is pressing esc, then quit
//		PostQuitMessage( 0 );
//		return false;
//		break;

	case WM_SYSCOMMAND:
		switch( wparam ) {
		case SC_SCREENSAVE:		//Is the computer trying to get a screen saver going?
		case SC_MONITORPOWER:	//Does the computer want to go to sleep? (I know I do :) )
			return false;
		}
		break;

	case WM_CLOSE:				//Does the user want to close?
		PostQuitMessage( 0 );		//Send a quit message to windows
		return false;

	case WM_KEYDOWN:
		keys[wparam] = true;
		return false;

	case WM_KEYUP:
		keys[wparam] = false;
		return false;

	case WM_SIZE:				//Resize the window (and then resize the current scene to fit
		ogl.Resize( LOWORD(lparam), HIWORD(lparam) );  // LOWORD=width, HIWORD=height
		return false;
	} 

	return DefWindowProc( hwnd, msg, wparam, lparam );
}

//modelIndex drawindex;
//textureIndex	tex;

drawInfo_t di;

SeeDHeightMap map;
SeeDFrustum   frustum;

textureIndex  tex;
//modelIndex    model;


gmVector3 pos(0.0f, -50.0f, 50.0f);
gmVector3 angles(0.0f, 0.0f, 0.0f);
gmVector3 axis[3] = { gmVector3(1, 0, 0),
                      gmVector3(0, 1, 0),
                      gmVector3(0, 0, 1) };


ulong	start_time, end_time;
float	tot_time;

#define START_TIME	(start_time = timeGetTime())
#define END_TIME	((end_time = timeGetTime()), (tot_time=(float)(end_time-start_time)/1000))

bool
CreateOGLWindow(uint mode, uint bpp, float fov, bool fullscreen)
{
	WindowInfo info;

	info.mode = mode;
	info.x = 0;
	info.y = 0;
	info.fullscreen = fullscreen;
	info.classname = CLASSNAME;
	info.title = TITLE;

	if( !ogl.CreateWin(info, fov) )
		return false;

	return true;
}

bool
Game_Init()
{

START_TIME;
	if( !CreateOGLWindow(3, 32, 45.0f, false) )
		return false;
END_TIME;

	logger.Printf("%f seconds to Create OGL Window", tot_time);

START_TIME;
	if( !seed.Initialize() )
		return false;
END_TIME;

	logger.Printf("%f seconds to Initialize the engine", tot_time);


#if 1
START_TIME;
	if( !map.Load("bin/textures/map.tga") )
		return false;
END_TIME;

	logger.Printf("%f seconds to Create Map", tot_time);
#endif

	angles[YAW] = 90.0f;
	AnglesToAxis(angles, axis);
	ogl.SetView(pos, angles);
	frustum.Calculate();

	memset( &di, 0, sizeof(drawInfo_t) );
	di.index = media.LoadModel("bin/models/head.md3");
	di.texture = media.LoadTexture("bin/textures/h_bitterman.tga");
	AxisClear(di.axis);
	di.pos[0] = -50.0f;

	tex = media.LoadTexture("bin/textures/h_bitterman.tga");

	seed.Draw( di, true );

#if 0
	model.Load("models/players/kyle/model.glm");
	animations.Load("models/players/_humanoid/_humanoid.gla");
	model.SetAnimations( &animations );
#endif

	return true;
}


unsigned long frames, time = 0, last_time, last_update = 0;
float msec, fps;

#define TURN_SCALE 45.0f
#define MOVE_SCALE 40.0f

bool update_frustum = true;

bool
HandleRotation()
{
	gmVector3 new_angles(0.0f, 0.0f, 0.0f);

	if( keys[VK_UP] )
		new_angles[PITCH] += TURN_SCALE;
	if( keys[VK_DOWN] )
		new_angles[PITCH] -= TURN_SCALE;
	if( keys[VK_LEFT] )
		new_angles[YAW] += TURN_SCALE;
	if( keys[VK_RIGHT] )
		new_angles[YAW] -= TURN_SCALE;

	if( dot(new_angles, new_angles) == 0.0f )
		return false;

    new_angles *= msec;
	angles += new_angles;

	AnglesToAxis( angles, axis );

	return true;
}

bool
HandleMovement()
{
	float forward = 0.0f, right = 0.0f, up = 0.0f;

	if( keys['W'] )
		forward += MOVE_SCALE;
	if( keys['S'] )
		forward -= MOVE_SCALE;
	if( keys['A'] )
		right += MOVE_SCALE;
	if( keys['D'] )
		right -= MOVE_SCALE;
	if( keys[VK_CONTROL] )
		up -= MOVE_SCALE;
	if( keys[VK_SPACE] )
		up += MOVE_SCALE;


	if( !right && !forward && !up )
		return false;

	if( right )
		VectorMA(pos, right*msec, axis[1], pos);
	if( forward )
		VectorMA(pos, forward*msec, axis[0], pos);
	if( up )
		VectorMA(pos, up*msec, axis[2], pos);

	return true;
}

void
UpdateView()
{
	if( HandleMovement() )
		update_frustum = true;

	if( HandleRotation() )
		update_frustum = true;

	if( update_frustum ) {
		ogl.SetView( pos, angles );
	}
}

bool	tog_tog=true;
ulong	l_tog=0;


bool
RunFrame()
{
	// takes care of the begin and end of frame calls
//	seed_render_object r_obj;
	if( keys[VK_ESCAPE] ) {
		PostQuitMessage( 0 );
	}

	if( keys[VK_F1] ) {
		keys[VK_F1] = false;
		ogl.ToggleFullscreen();
	}

	if( keys['t'] || keys['T'] ) {
		if( time > (l_tog+1000) ) {
			tog_tog = !tog_tog;
			l_tog = time;
		}
	}

	last_time = time;
	time = timeGetTime(); // Get current time in seconds
	++frames;

	// Update the frame rate once per second
	if( time-last_update > 1000 ) {
		fps			= (float)(1000*frames)/(float)(time-last_update);
		last_update	= time;
		frames		= 0;
	}

	msec = (time - last_time) / 1000.0f;

	ogl.BeginFrame();

	if( update_frustum )
		frustum.Calculate();
	update_frustum = false;

	// add drawing info to the list of items to draw
//	seed.Draw(di);

#if 0
	matrix4x4 mat;
	ogl.GetInverseModelView( mat );
	vec4_t lpos = { 0, 100, 500, 1 };
	MatrixVectorMultiply( mat, lpos, lpos );
	glProgramEnvParameter4fvARB(GL_VERTEX_PROGRAM_ARB, 0, lpos);
#else
	float lpos[4] = { 0, 100, 500, 1 };
	glProgramEnvParameter4fvARB(GL_VERTEX_PROGRAM_ARB, 0, lpos);
#endif

	seed.Draw();

//	animations.Draw( 0 );
//	model.Draw(50, 0);

	// render crap

//	glColor3f(0.5f,0.5f,1.0f);
//	glPushMatrix();
//	glLoadIdentity();
//	glTranslatef(0.0f,0.0f,16.0f);
/*	glBegin(GL_TRIANGLES);
		glColor3f(1.0f,0.0f,0.0f); glVertex3f(0.0f,  0.0f,  0.0f);
		glColor3f(0.0f,1.0f,0.0f); glVertex3f(0.0f, 10.0f, 10.0f);
		glColor3f(0.0f,0.0f,1.0f); glVertex3f(0.0f,-10.0f, 10.0f);
	glEnd();*/

//	glPopMatrix();

	uint num = map.Draw();

	glFlush();

	// input, sound, etc

	UpdateView();

	ogl.BeginOrtho();

//	seed.DrawImage(100, 10, 128, 128, 0, 0, 1, 1, tex);

//	seed.Draw_Text( 10, 10, "Num Nodes Drawn: %lu", num );
//	seed.Draw_Text( 10, 400, "%.4f %.4f %.4f", pos[0], pos[1], pos[2] );

	seed.Draw_Text( 10, 420, "%.4f", fps );

	glFinish();
	ogl.EndOrtho();

	return true;
}


bool
Game_Shutdown()
{
	LogMessage(" -Shutdown Started-");

	map.Unload();

	media.UnloadAll();
	seed.Shutdown();

//	ogl.DestroyGL();
	ogl.DestroyWin();

	return true;
}



int WINAPI
WinMain( HINSTANCE hInstance,
		 HINSTANCE hPrevInstance,
         LPSTR lpCmdLine,
         int nCmdShow )
{
	MSG msg;

/*	int flag = _CrtSetDbgFlag(_CRTDBG_REPORT_FLAG);
	flag |= _CRTDBG_LEAK_CHECK_DF;
	_CrtSetDbgFlag(flag);

	HANDLE file =	CreateFile("memory.txt",	// open MYFILE.TXT 
					GENERIC_WRITE,				// open for reading 
					FILE_SHARE_READ,			// share for reading 
					NULL,						// no security 
					OPEN_EXISTING,				// existing file only 
					FILE_ATTRIBUTE_NORMAL,		// normal file 
					NULL);						// no attr. template 

	_CrtSetReportMode(_CRT_WARN, _CRTDBG_MODE_FILE);
	_CrtSetReportFile(_CRT_WARN, file);
*/

	if( hPrevInstance )
		return 0;

	//Do all of the Initiation stuff
	if( !Game_Init() ) {
		Game_Shutdown();

		return E_FAIL;
	}

	while( TRUE ) {
		if( !minimized )
			Sleep(1);

		while( PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE) ) {
			if( !GetMessage(&msg, NULL, 0, 0) ) {
				// quit
				goto shutdown;
			}

//			if( msg.message == WM_QUIT )
//				return false;

			TranslateMessage( &msg );
			DispatchMessage ( &msg );
		}

		// 
		RunFrame();
	}

shutdown:
	//Call Game_Shutdown, and do all the shutdown procedures
	Game_Shutdown();

//	_CrtDumpMemoryLeaks();
//	CloseHandle(file);

	return 0;
}

