#ifndef _MD3_H_
#define _MD3_H_

#include "..\local.h"

#define MD3_IDENT			(('3'<<24)+('P'<<16)+('D'<<8)+'I')
#define MD3_VERSION			15

// limits
#define MD3_MAX_LODS		4
#define MD3_MAX_TRIANGLES	8192	// per surface
#define MD3_MAX_VERTS		4096	// per surface
#define MD3_MAX_SHADERS		256		// per surface
#define MD3_MAX_FRAMES		1024	// per model
#define MD3_MAX_SURFACES	32		// per model
#define MD3_MAX_TAGS		16		// per frame

// vertex scales
#define	MD3_XYZ_SCALE		(1.0/64)

typedef struct {
	vec3_t		xyz;		// 0f
	vec3_t		normal;		// 3f
	vec3_t		color;		// 6f
	float		st[2];		// 9f
	int			index;		// 11f
} baseVertex_t;				// 11f+1i
	
typedef struct {
	baseVertex_t	v[3];
} baseTriangle_t;

typedef struct md3Frame_s {
	vec3_t		bounds[2];
	vec3_t		localOrigin;
	float		radius;
	char		name[16];
} md3Frame_t;

typedef struct md3Tag_s {
	char		name[64];	// tag name
	vec3_t		origin;
	vec3_t		axis[3];
} md3Tag_t;


/*
** md3Surface_t
**
** CHUNK			SIZE
** header			sizeof( md3Surface_t )
** shaders			sizeof( md3Shader_t ) * numShaders
** triangles[0]		sizeof( md3Triangle_t ) * numTriangles
** st				sizeof( md3St_t ) * numVerts
** XyzNormals		sizeof( md3XyzNormal_t ) * numVerts * numFrames
*/
typedef struct {
	int		ident;				// 

	char	name[64];	// polyset name

	int		flags;
	int		numFrames;			// all surfaces in a model should have the same

	int		numShaders;			// all surfaces in a model should have the same
	int		numVerts;

	int		numTriangles;
	int		ofsTriangles;

	int		ofsShaders;			// offset from start of md3Surface_t
	int		ofsSt;				// texture coords are common for all frames
	int		ofsXyzNormals;		// numVerts * numFrames

	int		ofsEnd;				// next surface follows
} md3Surface_t;


typedef struct {
	char			name[64];
	int				shaderIndex;	// for in-game use
} md3Shader_t;

typedef struct {
	int			indexes[3];
} md3Triangle_t;


typedef struct {
	float		st[2];
} md3St_t;


typedef struct {
	short		xyz[3];
	short		normal;
} md3XyzNormal_t;


typedef struct {
	int			ident;
	int			version;

	char		name[64];	// model name

	int			flags;

	int			numFrames;
	int			numTags;			
	int			numSurfaces;

	int			numSkins;

	int			ofsFrames;			// offset for first frame
	int			ofsTags;			// numFrames * numTags
	int			ofsSurfaces;		// first surface, others follow

	int			ofsEnd;				// end of file
} md3Header_t;


typedef struct md3SurfaceData_s {
	md3Surface_t	header;
	md3Shader_t		*shaders;//[MD3_MAX_SHADERS];
	// all verts
	float			**verts;//[MD3_MAX_FRAMES];

	baseTriangle_t	*baseTriangles;//[MD3_MAX_TRIANGLES];

	// sorted so that they form long tristrips
	int				*orderedTriangles;//[MD3_MAX_TRIANGLES][3];
	int				*lodTriangles;//[MD3_MAX_TRIANGLES][3];
	baseVertex_t	*baseVertices;//[MD3_MAX_VERTS];
} md3SurfaceData_t;


typedef struct md3Data_s {
	int					skinwidth, skinheight;

	md3SurfaceData_t	*surfData;//[MD3_MAX_SURFACES];
	md3Tag_t			*tags;//[MD3_MAX_FRAMES][MD3_MAX_TAGS];
	md3Frame_t			*frames;//[MD3_MAX_FRAMES];

	md3Header_t			*model;
	float				scale_up;
	vec3_t				adjust;
	vec3_t				aseAdjust;
	int					fixedwidth, fixedheight;

	int					maxSurfaceTris;

	int					lowerSkipFrameStart, lowerSkipFrameEnd;
	int					maxUpperFrames;
	int					maxHeadFrames;
	int					currentLod;
	float				lodBias;
	
	int					type;
} md3Data_t;



//----------------------------------------------|--



typedef struct md3Vertex_s {
	signed short	vertices[3];
	byte			envVertices[2];
} md3Vertex;

typedef struct md3Header_s {
	char		headerID[4];
	int			version;
	char		fileName[68];
	int			numFrames;
	int			numTags;
	int			numMeshes;
	int			numTextures;
	int			frameStart;
	int			tagStart;
	int			meshStart;
	int			fileSize;
} md3Header;

typedef struct md3MeshHeader_s {
	char		headerID[4];
	char		name[68];
	int			numFrames;
	int			numTextures;
	int			numVertices;
	int			numFaces;
	int			startFaces;
	int			startTextures;
	int			startTexVertices;
	int			startVertices;
	int			sizeMesh;
} md3MeshHeader;

typedef struct s_md3Tag {
	char		name[64];
	float		position[3];
	float		rotation[3][3];
} md3Tag;

typedef struct md3BoneFrame_s {
	float		min[3];
	float		max[3];
	float		position[3];
	float		scale;
	char		parent[16];
} md3BoneFrame;

typedef struct md3MeshFrame_s {
	md3Vertex	*vertices;
} md3MeshFrame;

typedef float	float2[2];
typedef int		int3[3];

typedef struct md3Mesh_s {
	char			meshname[68];	// object name

	int				numFrames;		// number of frames
	int				numTextures;	// number of textures
	int				numFaces;		// number of faces
	int				numVertex;		// number of vertices

	char			*frames;		// list of vertices
	float2			*texcoords;		// list of texture coordinates
	int3			*triangles;		// list of faces
	char			textures[68];	// texture names
} md3Mesh;

/*
class md3data {
public:
	int		numBoneFrames;
	int		numTags;
	int		numMeshes;
	int		numTextures;

	md3BoneFrame	*boneFrames;
	md3Tag			*tags;
	md3Mesh			*meshes;

	md3() {
		boneFrames=NULL;
		tags=NULL;
		meshes=NULL;
	}

	~md3();
};*/


#endif

