
#ifndef MDX_MODEL_H
#define MDX_MODEL_H

#include "mdx/mdx_format.h"
#include "seed_model.h"

typedef struct mdx_vertex_s {
	vec3_t	normal;
	st_t	texCoords;
	vec3_t	position;

	float	weights[iMAX_G2_BONEWEIGHTS_PER_VERT];
} mdx_vertex_t;

typedef struct mdx_surface_info_s {
	char	name[MAX_QPATH];
	uint	flags;
	char	shader[MAX_QPATH];
	int		shaderIndex;
	int		parentIndex;
	int		numChildren;
	int		*childIndexes;
} mdx_surface_info_t;	// struct size = (int)( &((mdxmSurfHierarch_t *)0)->childIndexes[ mdxmSurfHierarch_t->numChildren ] );


typedef struct mdx_surface_s {
	int			index;

	mdxmVertex_t	*verts;
	mdxmVertex_t	*blended_verts;
	st_t			*tex_coords;
	int				num_verts;

	tri_t		*tris;
	int			num_tris;

	int			*bone_refs;
	int			num_bone_refs;

	int			vertex_buffer;
} mdx_surface_t;

typedef struct mdx_lod_s {
	mdx_surface_t	*surfaces;
} mdx_lod_t;


class mdxm_animations;

class mdxm_model {
public:
	mdxm_model()
		:	surf_data(NULL), LODs(NULL),
			num_surfaces(0), num_lods(0), valid(false)
	{ }

public:
	bool Load( char *filename );

	bool SetAnimations( mdxm_animations *anims )
	{ if( !anims ) return false; _anims = anims; return true; }

	void Draw( int frame, int lod=0 );

	void Unload();

protected:
	bool DrawSurface( mdx_surface_t *surface );

protected:
	mdx_surface_info_t	*surf_data;
	mdx_lod_t			*LODs;

	int				num_lods;
	int				num_surfaces;
	int				root_num;

	mdxm_animations	*_anims;

	bool			valid;
protected:
	mdx_surface_t	*working_surfaces;
	int				working_frame;
};

//
typedef struct bone_s {
//	matrix3x4	matrix;
	quaternion	axis;
	vec3_t		pos;
} bone_t;

typedef struct mdx_skeleton_s {
	char		name[MAX_QPATH];
	uint		flags;
	int			parent;
	bone_t		BasePoseMat;
	bone_t		BasePoseMatInv;
	int			numChildren;
	int			*children;
} mdx_skeleton_t;

typedef struct mdx_frame_s {
	int			*index;
} mdx_frame_t;


class mdxm_animations {
public:
	mdxm_animations()
		:	frames(NULL), skeletons(NULL), bones(NULL),
			num_frames(0), num_bones(0), num_quats(0)
	{ }

	friend mdxm_model;

public:
	bool	Load( char *filename );

	void	DumpMDXA();
	void	Unload();

	void	Draw( int frame, int bone=-1 );

protected:
	mdx_skeleton_t	*GetBoneInfo( int bone_id );
	bone_t			*GetChunkBone( int bone_id, int frame );

	bool			SetWorkingFrame( int frame, int bone=-1 );

	bone_t			*GetRoot();

protected:

	mdx_frame_t		*frames;
	mdx_skeleton_t	*skeletons;
	bone_t			*bones;

	int			root_num;

	int			num_frames;
	int			num_bones;
	int			num_quats;

	int				working_frame;
	bone_t			*working_bones;
};

#endif
