
#include "seed_ogl.h"
#include <stdio.h>
#include <io.h>

//

bool SeeDOpenGL::CheckExtension( char *ext ) {
	char	*p = (char*)config.extensions_string;
	char	*end = p + strlen(p);

	int	ext_len = strlen(ext);

	while( p < end) {
		int n = strcspn(p, " ");

		if( ext_len == n && !strncmp(ext, p, n) )
			return true;

		p += (n + 1);
	}

	return false;
}




const char DefaultShader [] = {
	"!!VP1.0 "
	// Apply the modelview matrix
	"DP4 R0.x, v[OPOS], c[0];"	
	"DP4 R0.y, v[OPOS], c[1];"
	"DP4 R0.z, v[OPOS], c[2];"
	"DP4 R0.w, v[OPOS], c[3];"
	// Apply the projection matrix
	"DP4 R1.x, R0, c[4];"
	"DP4 R1.y, R0, c[5];"
	"DP4 R1.z, R0, c[6];"
	"DP4 R1.w, R0, c[7];"
	"MOV o[HPOS], R1;"
	"MOV o[TEX0], v[TEX0];"
	"END"
};

const char *ExampleProgram = {
	"!!ARBvp1.0"

	"#Input"
	"ATTRIB InPos = vertex.position;"
	"ATTRIB InColor = vertex.color;"

	"#Output"
	"OUTPUT OutPos = result.position;"
	"OUTPUT OutColor = result.color;"

	"PARAM MVP[4] = { state.matrix.mvp }; # Modelview Projection Matrix."
	"TEMP Temp;"

	"#Transform vertex to clip space"
	"DP4 Temp.x, MVP[0], InPos;"
	"DP4 Temp.y, MVP[1], InPos;"
	"DP4 Temp.z, MVP[2], InPos;"
	"DP4 Temp.w, MVP[3], InPos;"

	"#Output"
	"MOV OutPos, temp;"
	"MOV OutColor, InColor;"

	"END"
};

const char ARBVP[] = 
	"!!ARBvp1.0\
	#\n\
	# This is a sample ARB Vertex Program\n\
	#\n\
	ATTRIB iPos         = vertex.position;\
	ATTRIB iColor		= vertex.color;\
	PARAM  mvp[4]       = { state.matrix.mvp };\
	OUTPUT oPos         = result.position;\
	OUTPUT oColor       = result.color;\
	# This just does out vertex transform by the modelview projection matrix\n\
	DP4 oPos.x, mvp[0], iPos;\
	DP4 oPos.y, mvp[1], iPos;\
	DP4 oPos.z, mvp[2], iPos;\
	DP4 oPos.w, mvp[3], iPos;\
	# Write our color out directly\n\
	MOV oColor, iColor;\
	END";

bool have_vert_programARB = false;

void SetupVertexProgramARB() {
	if( have_vert_programARB ) return;

	if( !ogl.CheckExtension("GL_ARB_vertex_program") )	return;

	have_vert_programARB = true;
}

class vertex_programmerARB {
public:
	vertex_programmerARB() { }

	uint LoadProgram( const char *program ) {
		uint	vp;

		glGenProgramsARB(1, &vp);
		glBindProgramARB(GL_VERTEX_PROGRAM_ARB, vp);

		glProgramStringARB(GL_VERTEX_PROGRAM_ARB, GL_PROGRAM_FORMAT_ASCII_ARB, strlen(program), program);

		//Load the Light direction into Vector Register c[8].
//		glProgramEnvParameter4fARB(GL_VERTEX_PROGRAM_ARB, 8,  1.0f, -0.1f, 5, 1);	// Light direction
//		glProgramEnvParameter4fARB(GL_VERTEX_PROGRAM_ARB, 9,  0.8f, 0.8f, 0.8f, 1);	// Light color

		return vp;
	}

	uint LoadProgramFromFile( char *filename ) {
		FILE *file = fopen(filename, "rt");

		if( !file ) {
			return LoadProgram( (char*)DefaultShader );
		} else {
			uint fs = filelength( fileno(file) );
			char *program = new char[fs];
			fread(program, fs, 1, file);
			uint vp = LoadProgram( program );
//			delete [] program;

			return vp;
		}
	}

	void UnloadProgram( uint program ) {
		glDeleteProgramsARB( 1, &program );
	}

	void EnableProgram( uint program ) {
		return;

		glBindProgramARB(GL_VERTEX_PROGRAM_ARB, program);
		glEnable(GL_VERTEX_PROGRAM_ARB);
	}

	void DisableProgram() {
		return;

		glDisable(GL_VERTEX_PROGRAM_ARB);
	}

protected:
};


//
// Compiled Vertex Arrays

bool	have_comp_vert_arrayEXT = false;

void SetupCompiledVertexArrayEXT() {
	if( have_comp_vert_arrayEXT) return;

	if( !ogl.CheckExtension("GL_EXT_compiled_vertex_array") ) return;

	have_comp_vert_arrayEXT = true;
}

//
// Vertex Buffer Objects

bool	have_vert_buff_obectARB = false;


void SetupVertexBufferObjectARB() {
	if( have_vert_buff_obectARB ) return;

	if( !ogl.CheckExtension("GL_ARB_vertex_buffer_object") ) return;

	have_vert_buff_obectARB = true;
}

//
//

bool	have_occlusion_queryNV = false;


void SetupOcclusionQueryNV() {
	if( have_occlusion_queryNV ) return;

	if( !ogl.CheckExtension("GL_NV_occlusion_query") ) return;

	have_occlusion_queryNV = true;
}


//
// GL_ARB_vertex_blend

bool have_vertex_blendARB=false;



void SetupVertexBlendARB() {
	if( have_vertex_blendARB ) return;

	if( !ogl.CheckExtension("GL_ARB_vertex_blend") ) return;

	have_vertex_blendARB = true;
}

//


bool	have_bgraEXT=false;
bool	have_tex_compression=false;
bool	have_mipmap_gen=false;


bool
SeeDOpenGL::InitExtensions()
{

    GLenum error = glewInit();
    if( error != GLEW_OK )
    {
        return false;
    }

	SetupVertexProgramARB();
	SetupCompiledVertexArrayEXT();
	SetupVertexBufferObjectARB();
	SetupVertexBlendARB();

	SetupOcclusionQueryNV();

	have_bgraEXT			= ogl.CheckExtension("GL_EXT_bgra");
	have_tex_compression	= ogl.CheckExtension("GL_ARB_texture_compression");
	have_mipmap_gen			= ogl.CheckExtension("GL_SGIS_generate_mipmap");

	return (have_comp_vert_arrayEXT && have_vert_programARB && have_vert_buff_obectARB);
}

