
#include "seed_math.h"
#include <memory.h>
#include <math.h>

//#include "mmgr.h"

void
AxisClear(gmVector3 axis[3])
{
    axis[0].assign(1, 0, 0);
    axis[1].assign(0, 1, 0);
    axis[2].assign(0, 0, 1);
}


void
AxisCopy(const gmVector3 in[3], gmVector3 out[3])
{
    out[0].assign(in[0]);
    out[1].assign(in[1]);
    out[2].assign(in[2]);
}


void MatrixMultiply( float in1[3][3], float in2[3][3], float out[3][3] ) {
	out[0][0] = in1[0][0] * in2[0][0] + in1[0][1] * in2[1][0] + in1[0][2] * in2[2][0];
	out[0][1] = in1[0][0] * in2[0][1] + in1[0][1] * in2[1][1] + in1[0][2] * in2[2][1];
	out[0][2] = in1[0][0] * in2[0][2] + in1[0][1] * in2[1][2] + in1[0][2] * in2[2][2];

	out[1][0] = in1[1][0] * in2[0][0] + in1[1][1] * in2[1][0] + in1[1][2] * in2[2][0];
	out[1][1] = in1[1][0] * in2[0][1] + in1[1][1] * in2[1][1] + in1[1][2] * in2[2][1];
	out[1][2] = in1[1][0] * in2[0][2] + in1[1][1] * in2[1][2] + in1[1][2] * in2[2][2];

	out[2][0] = in1[2][0] * in2[0][0] + in1[2][1] * in2[1][0] + in1[2][2] * in2[2][0];
	out[2][1] = in1[2][0] * in2[0][1] + in1[2][1] * in2[1][1] + in1[2][2] * in2[2][1];
	out[2][2] = in1[2][0] * in2[0][2] + in1[2][1] * in2[1][2] + in1[2][2] * in2[2][2];
}

// column major

void MatrixMultiply( const matrix4x4 in1, const matrix4x4 in2, matrix4x4 out ) {
	out[ 0] = in1[ 0] * in2[ 0] + in1[ 1] * in2[ 4] + in1[ 2] * in2[ 8] + in1[ 3] * in2[12];
	out[ 1] = in1[ 0] * in2[ 1] + in1[ 1] * in2[ 5] + in1[ 2] * in2[ 9] + in1[ 3] * in2[13];
	out[ 2] = in1[ 0] * in2[ 2] + in1[ 1] * in2[ 6] + in1[ 2] * in2[10] + in1[ 3] * in2[14];
	out[ 3] = in1[ 0] * in2[ 3] + in1[ 1] * in2[ 7] + in1[ 2] * in2[11] + in1[ 3] * in2[15];

	out[ 4] = in1[ 4] * in2[ 0] + in1[ 5] * in2[ 4] + in1[ 6] * in2[ 8] + in1[ 7] * in2[12];
	out[ 5] = in1[ 4] * in2[ 1] + in1[ 5] * in2[ 5] + in1[ 6] * in2[ 9] + in1[ 7] * in2[13];
	out[ 6] = in1[ 4] * in2[ 2] + in1[ 5] * in2[ 6] + in1[ 6] * in2[10] + in1[ 7] * in2[14];
	out[ 7] = in1[ 4] * in2[ 3] + in1[ 5] * in2[ 7] + in1[ 6] * in2[11] + in1[ 7] * in2[15];

	out[ 8] = in1[ 8] * in2[ 0] + in1[ 9] * in2[ 4] + in1[10] * in2[ 8] + in1[11] * in2[12];
	out[ 9] = in1[ 8] * in2[ 1] + in1[ 9] * in2[ 5] + in1[10] * in2[ 9] + in1[11] * in2[13];
	out[10] = in1[ 8] * in2[ 2] + in1[ 9] * in2[ 6] + in1[10] * in2[10] + in1[11] * in2[14];
	out[11] = in1[ 8] * in2[ 3] + in1[ 9] * in2[ 7] + in1[10] * in2[11] + in1[11] * in2[15];

	out[12] = in1[12] * in2[ 0] + in1[13] * in2[ 4] + in1[14] * in2[ 8] + in1[15] * in2[12];
	out[13] = in1[12] * in2[ 1] + in1[13] * in2[ 5] + in1[14] * in2[ 9] + in1[15] * in2[13];
	out[14] = in1[12] * in2[ 2] + in1[13] * in2[ 6] + in1[14] * in2[10] + in1[15] * in2[14];
	out[15] = in1[12] * in2[ 3] + in1[13] * in2[ 7] + in1[14] * in2[11] + in1[15] * in2[15];
}

void MatrixAdd( const matrix4x4 in1, const matrix4x4 in2, matrix4x4 out ) {
	out[ 0] = in1[ 0]+in2[ 0]; out[ 1] = in1[ 1]+in2[ 1]; out[ 2] = in1[ 2]+in2[ 2]; out[ 3] = in1[ 3]+in2[ 3];
	out[ 4] = in1[ 4]+in2[ 4]; out[ 5] = in1[ 5]+in2[ 5]; out[ 6] = in1[ 6]+in2[ 6]; out[ 7] = in1[ 7]+in2[ 7];
	out[ 8] = in1[ 8]+in2[ 8]; out[ 9] = in1[ 9]+in2[ 9]; out[10] = in1[10]+in2[10]; out[11] = in1[11]+in2[11];
	out[12] = in1[12]+in2[12]; out[13] = in1[13]+in2[13]; out[14] = in1[14]+in2[14]; out[15] = in1[15]+in2[15];
}

void MatrixVectorMultiply( const matrix4x4 mat, const gmVector4& in, gmVector4& out ) {
	out[0] = in[0]*mat[0] + in[0]*mat[4] + in[0]*mat[ 8] + in[0]*mat[12];
	out[1] = in[1]*mat[1] + in[1]*mat[5] + in[1]*mat[ 9] + in[1]*mat[13];
	out[2] = in[2]*mat[2] + in[2]*mat[6] + in[2]*mat[10] + in[2]*mat[14];
	out[3] = in[3]*mat[3] + in[3]*mat[7] + in[3]*mat[11] + in[3]*mat[15];
}


void
AngleVectors(const gmVector3& angles,
             gmVector3& forward,
             gmVector3& right,
             gmVector3& up)
{
    float  angle;
    static float sr, sp, sy, cr, cp, cy;
    // static to help MS compiler fp bugs

    angle = angles[YAW] * (S_PI*2 / 360);
    sy = sin(angle);
    cy = cos(angle);
    angle = angles[PITCH] * (S_PI*2 / 360);
    sp = sin(angle);
    cp = cos(angle);
    angle = angles[ROLL] * (S_PI*2 / 360);
    sr = sin(angle);
    cr = cos(angle);

    forward.assign(cp*cy,
                   cp*sy,
                   -sp);

    right.assign(-1*sr*sp*cy+-1*cr*-sy,
                 -1*sr*sp*sy+-1*cr*cy,
                 -1*sr*cp);

    up.assign(cr*sp*cy+-sr*-sy,
              cr*sp*sy+-sr*cy,
              cr*cp);
}

void
AnglesToAxis(const gmVector3& angles, gmVector3 axis[3])
{
	gmVector3 right;
	gmVector3 origin(0, 0, 0);

	// angle vectors returns "right" instead of "y axis"
	AngleVectors(angles, axis[0], right, axis[2]);
    axis[1].assign(origin - right);
}


