#ifndef S_SEED_MATH_H
#define S_SEED_MATH_H

#include <gmVec3.h>
#include <gmVec4.h>

#include "seed_types.h"

#pragma warning(disable : 4244)		// conversion from double to float

#define PITCH	0
#define YAW		1
#define ROLL	2

#define MIN(a, b)  (((a) < (b)) ? (a) : (b))
#define MAX(a, b)  (((a) > (b)) ? (b) : (a))

#define RANDOM_FLOAT			(((float)rand()-(float)rand())/RAND_MAX)
#define CHECK_RANGE(x,min,max)	((x)=((x)<(min)?(min):(x)<(max)?(x):(max)))
#define DEG_TO_RAD(angle)		((angle)*PI/180)
#define RAD_TO_DEG(radians)		((radians)*180/PI)
#define SQUARE(number)			((number)*(number))

#define	S_PI		3.14159265358979323846
#define DEG2RAD(a)	( ((a)*S_PI)/180.0f )
#define RAD2DEG(a)	( ((a)*180.0f)/S_PI )


//inline float fabs(const float num) { return ( num < 0.0f ) ? 0.0f - num : num; }

#define DotProduct(x,y)			((x)[0]*(y)[0]+(x)[1]*(y)[1]+(x)[2]*(y)[2])
#define DotProduct4(x,y)		((x)[0]*(y)[0]+(x)[1]*(y)[1]+(x)[2]*(y)[2]+(x)[3]*(y)[3])
#define VectorSubtract(a,b,c)	((c)[0]=(a)[0]-(b)[0],(c)[1]=(a)[1]-(b)[1],(c)[2]=(a)[2]-(b)[2])
#define VectorAdd(a,b,o)		((o)[0]=(a)[0]+(b)[0],(o)[1]=(a)[1]+(b)[1],(o)[2]=(a)[2]+(b)[2])
#define VectorAdd2(a,b,c,o)		((o)[0]=(a)[0]+(b)[0]+(c)[0],(o)[1]=(a)[1]+(b)[1]+(c)[1],(o)[2]=(a)[2]+(b)[2]+(c)[2])
#define VectorAdd3(a,b,c,d,o)	((o)[0]=(a)[0]+(b)[0]+(c)[0]+(d)[0],(o)[1]=(a)[1]+(b)[1]+(c)[1]+(d)[1],(o)[2]=(a)[2]+(b)[2]+(c)[2]+(d)[2])

#define VectorCopy(a,b)			((b)[0]=(a)[0],(b)[1]=(a)[1],(b)[2]=(a)[2])
#define	VectorScale(v, s, o)	((o)[0]=(v)[0]*(s),(o)[1]=(v)[1]*(s),(o)[2]=(v)[2]*(s))
#define	VectorScale4(v, s, o)	((o)[0]=(v)[0]*(s),(o)[1]=(v)[1]*(s),(o)[2]=(v)[2]*(s),(o)[3]=(v)[3]*(s))
#define	VectorMA(v, s, b, o)	((o)[0]=(v)[0]+(b)[0]*(s),(o)[1]=(v)[1]+(b)[1]*(s),(o)[2]=(v)[2]+(b)[2]*(s))

#define VectorClear(a)			((a)[0]=(a)[1]=(a)[2]=0)
#define VectorNegate(a,b)		((b)[0]=-(a)[0],(b)[1]=-(a)[1],(b)[2]=-(a)[2])
#define VectorSet(v,x,y,z)		((v)[0]=(x), (v)[1]=(y), (v)[2]=(z))
#define VectorSet4(v,x,y,z,w)	((v)[0]=(x), (v)[1]=(y), (v)[2]=(z), (v)[3]=(w))
#define VectorCopy4(a,b)		((b)[0]=(a)[0],(b)[1]=(a)[1],(b)[2]=(a)[2],(b)[3]=(a)[3])

#define VectorLength(v)			( gmSqrt(DotProduct((v),(v))) )

#define DistanceToPoint(p, v)	( DotProduct((p),(v))+(p)[3] )


#define	QuatAdd(a,b,o)			((o)[0]=(a)[0]+(b)[0],(o)[1]=(a)[1]+(b)[1],(o)[2]=(a)[2]+(b)[2],(o)[3]=(a)[3]+(b)[3])
#define	QuatInverse(q, p)		(VectorScale((q), -1.0f, (p)), (p)[3]=(q)[3])
#define	QuatConjugate(q)		((vec_t)sqrt(DotProduct4((q),(q))))
#define	QuatSet(q, x, y, z, w)	((q)[0]=(x),(q)[1]=(y),(q)[2]=(z),(q)[3]=(w))

#define	QuatNormalize(q)	(VectorNormalize4((q)))

/*
void	QuatMultiply( const quaternion a, const quaternion b, quaternion out );
void	QuatSlerp( quaternion from, quaternion to, float t, quaternion res );
void	QuatVec3Mult( const quaternion quat, const vec3_t in, vec3_t out );
void	QuatToMatrix3x3( const quaternion quat, matrix3x3 mat );
void	QuatToMatrix4x4( const quaternion quat, matrix4x4 mat );
void	AnglesToQuat( const vec3_t angles, quaternion quat );
void	Matrix4x4ToQuat( const matrix3x4 mat, quaternion quat );
void	Matrix3x4ToQuat( const matrix3x4 mat, quaternion quat );*/

#define CrossProduct(v1, v2, o)	((o)[0]=(v1)[1]*(v2)[2]-(v1)[2]*(v2)[1], (o)[1]=(v1)[2]*(v2)[0]-(v1)[0]*(v2)[2], (o)[2]=(v1)[0]*(v2)[1]-(v1)[1]*(v2)[0])

void AxisClear(gmVector3 axis[3]);
void AxisCopy(const gmVector3 in[3], gmVector3 out[3]);
void MatrixMultiply(float in1[3][3], float in2[3][3], float out[3][3]);
void MatrixMultiply( const matrix4x4 in1, const matrix4x4 in2, matrix4x4 out );
void MatrixVectorMultiply( const matrix4x4 mat, const gmVector4& in, gmVector4& out );


void AngleVectors(const gmVector3& angles, gmVector3& forward, gmVector3& right, gmVector3& up);
void AnglesToAxis(const gmVector3& angles, gmVector3 axis[3]);


inline gmType PlaneNormalize(gmVector4& p) {
	gmType length, ilength;

	length = p[0]*p[0] + p[1]*p[1] + p[2]*p[2];
	length = (gmType)sqrt(length);

	if ( length ) {
		ilength = 1./length;
		p[0] *= ilength;
		p[1] *= ilength;
		p[2] *= ilength;
		p[3] *= ilength;
	}

	return length;
}



//void CrossProduct( const vec3_t v1, const vec3_t v2, vec3_t cross );


#endif
