
#include "SeeDOpenGL.h"
#include "LogManager.h"
#include "SeeDMedia.h"

#include <jpeglib.h>

//#include "mmgr.h"

SeeDMedia media;

SeeDMedia::SeeDMedia()
{
    this->modelList.clear();
    this->textureList.clear();
}


SeeDMedia::~SeeDMedia()
{
    this->UnloadAll();
}


bool
SeeDMedia::Init()
{
    this->UnloadAll();
    
    SeeDModel *model = new SeeDNullModel;
    this->modelList.push_back(model);
    this->nullModelIndex = 0;
    
    this->nullTextureIndex = 0;
    this->nullTextureIndex = LoadTexture("bin/textures/tex.tga");
    return true;
}


bool
SeeDMedia::UnloadAll()
{
    int i;
    int num = this->modelList.size();

	for( i=0; i<num; i++ )
		SAFE_DELETE(this->modelList[i]);
    this->modelList.clear();

    if( num )
		LogMessage("Removed %i Models", num);

    num = this->textureList.size();
	for( i=0; i<num; i++ ) {
        this->imageLoader.Unload(this->textureList[i]);
        this->textureList[i] = NULL;
    }
    this->textureList.clear();

	if( num )
		LogMessage("Removed %i Textures", num);


	return true;
}


modelIndex
SeeDMedia::LoadModel( char *filename )
{
	int i;
    const int num = this->modelList.size();

    // see if the model is loaded
	for( i=0; i<num; i++ )
        if( !_stricmp(this->modelList[i]->GetName(), filename) )
			break;

	if( i == num ) {
		SeeDModel *temp = this->modelLoader.LoadModel(filename);

		if( temp )
            this->modelList.push_back(temp);
		else
			i = this->nullModelIndex;
	}

	return i;
}


textureIndex
SeeDMedia::LoadTexture( char *filename )
{
	if( !filename )
		return this->nullTextureIndex;

    int i;
    const int num = this->textureList.size();

    // see if the texture is loaded
	for( i=this->nullTextureIndex; i<num; i++ )
		if( !_stricmp(this->textureList[i]->GetName(), filename) )
			break;

	if( i == num )
    {
		SeeDTexture* temp = this->imageLoader.Load(filename);

		if( temp )
			this->textureList.push_back( temp );
		else
			i = this->nullTextureIndex;
	}

	return i;
}


void
SeeDMedia::Draw( modelIndex mIndex )
{
    SeeDModel *temp = this->modelList[mIndex];

    if( !temp->IsLoaded() )
        return;

    drawInfo_t	di;
    memset( &di, 0, sizeof(drawInfo_t) );
    AxisClear( di.axis );

    temp->Draw(di);
}


void
SeeDMedia::Draw( drawInfo_t & di )
{
    if( di.index < 0 ||
        di.index > this->modelList.size() )
		return;

	if( di.texture >= 0 ) {
		glEnable(GL_TEXTURE_2D);
        this->textureList[ di.texture ]->Bind();
	} else {
		glDisable(GL_TEXTURE_2D);
	}

	SeeDModel* model = this->GetModelFromIndex(di.index);
	if( !model->Draw(di) ) {
        model = this->modelList[this->nullModelIndex];
		model->Draw(di);
	}
}


SeeDTexture*
SeeDMedia::GetTexture( textureIndex ind )
{
    if( ind >= this->textureList.size() )
        return this->textureList[this->nullTextureIndex];

	return this->textureList[ind];
}


SeeDModel*
SeeDMedia::GetModelFromIndex( modelIndex index )
{
    if( index < 0 ||
        index > this->modelList.size() )
    {
		index = this->nullModelIndex;
    }
	return this->modelList[index];
}


SeeDTexture*
SeeDMedia::GetTextureFromIndex( textureIndex index )
{
	if( index < 0 ||
        index > this->textureList.size() )
    {
		index = this->nullTextureIndex;
    }
	return this->textureList[index];
}



