
#include "seed_math.h"

//#include "mmgr.h"

#define DELTA 1e-6     // error tolerance


void matrix4x4_quat( matrix4x4 mat, quaternion quat ) {
}

/*
out.w = (a.v dot b.v)
out.v = (a.v cross b.v) + (a.v * b.w) + (a.w * b.v)
*/
void QuatMultiply( const quaternion a, const quaternion b, quaternion out ) {
	vec3_t	temp;

	out[3] = DotProduct(a, b);
	CrossProduct( a, b, out );
	VectorScale( a, b[3], temp );
	VectorAdd( out, temp, out );
	VectorScale( b, a[3], temp );
	VectorAdd( out, temp, out );

	QuatNormalize(out);
}

void QuatVec3Mult( const quaternion quat, const vec3_t in, vec3_t out ) {
	vec3_t in1;
	matrix3x3 mat;
	VectorCopy(in, in1);
	QuatToMatrix3x3( quat, mat );
	out[0] = in1[0]*mat[0] + in1[1]*mat[3] + in1[2]*mat[6];
	out[1] = in1[0]*mat[1] + in1[1]*mat[4] + in1[2]*mat[7];
	out[2] = in1[0]*mat[2] + in1[1]*mat[5] + in1[2]*mat[8];
}


void AnglesToQuat( const vec3_t angles, quaternion quat ) {
	float	angle;
	vec_t	cr, cp, cy, sr, sp, sy, cpcy, spsy;


	angle = angles[ROLL] * (S_PI*2 / 360);
	cr = cos(angle);
	sr = sin(angle);

	angle = angles[PITCH] * (S_PI*2 / 360);
	cp = cos(angle);
	sp = sin(angle);

	angle = angles[YAW] * (S_PI*2 / 360);
	cy = cos(angle);
	sy = sin(angle);

	cpcy = cp * cy;
	spsy = sp * sy;

	quat[3] = cr * cpcy + sr * spsy;
	quat[0] = sr * cpcy - cr * spsy;
	quat[1] = cr * sp * cy + sr * cp * sy;
	quat[2] = cr * cp * sy - sr * sp * cy;
}


/*
inv(a) = a'/(a*a')

void QuatInverse( const quaternion in, quaternion out ) {
	quaternion conj;
	QuatInverse(in, out);

}*/

void QuatToMatrix4x4( const quaternion quat, matrix4x4 mat ) {
	vec_t	xx = quat[0]*quat[0],
			yy = quat[1]*quat[1],
			zz = quat[2]*quat[2],

			xy = quat[0]*quat[1],
			xz = quat[0]*quat[2],
			yz = quat[1]*quat[2],

			xw = quat[0]*quat[3],
			yw = quat[1]*quat[3],
			zw = quat[2]*quat[3];

	mat[0] = 1.0f - 2.0f * (yy + zz);
	mat[4] = 2.0f * (xy - zw);
	mat[8] = 2.0f * (xz + yw);
	mat[12]= 0.0f;

	mat[1] = 2.0f * (xy + zw);
	mat[5] = 1.0f - 2.0f * (xx - zz);
	mat[9] = 2.0f * (yz - xw);
	mat[13]= 0.0f;

	mat[2] = 2.0f * (xz - yw);
	mat[6] = 2.0f * (yz + xw);
	mat[10]= 1.0f - 2.0f * (xx + yy);
	mat[14]= 0.0f;

	mat[3] = 0.0f;
	mat[7] = 0.0f;
	mat[11]= 0.0f;
	mat[15]= 1.0f;
}

void QuatToMatrix3x3( const quaternion quat, matrix3x3 mat ) {
	vec_t	xx = quat[0]*quat[0],
			yy = quat[1]*quat[1],
			zz = quat[2]*quat[2],

			xy = quat[0]*quat[1],
			xz = quat[0]*quat[2],
			yz = quat[1]*quat[2],

			xw = quat[0]*quat[3],
			yw = quat[1]*quat[3],
			zw = quat[2]*quat[3];

	mat[0] = 1.0f - 2.0f * (yy + zz);
	mat[3] = 2.0f * (xy - zw);
	mat[6] = 2.0f * (xz + yw);

	mat[1] = 2.0f * (xy + zw);
	mat[4] = 1.0f - 2.0f * (xx - zz);
	mat[7] = 2.0f * (yz - xw);

	mat[2] = 2.0f * (xz - yw);
	mat[5] = 2.0f * (yz + xw);
	mat[8]= 1.0f - 2.0f * (xx + yy);
}

void Matrix4x4ToQuat( const matrix4x4 mat, quaternion quat ) {
	vec_t trace = mat[0] + mat[5] + mat[10] + 1.0f;
	vec_t s;

	if( trace > 0.0f ) {
		// 'Instant' calculation
		s = 0.5  / (vec_t)sqrt(trace);
		quat[3] = 0.25 / s;
		quat[0] = (mat[6] - mat[9]) * s;
		quat[1] = (mat[8] - mat[2]) * s;
		quat[2] = (mat[1] - mat[4]) * s;
	} else {
		int n = 0;
		if( mat[5] > mat[0] )
			n = 1;
		if( mat[10] > mat[n*5] )
			n = 2;
		switch(n) {
		case 0:
			s = 1 / (sqrt(1.0f + mat[0] - mat[5] - mat[10]) * 2.0f);
			QuatSet(quat,
					0.5f * s,
					(mat[1] + mat[4]) * s,
					(mat[2] + mat[8]) * s,
					(mat[6] + mat[9]) * s);
			break;
		case 1:
			s = 1 / (sqrt(1.0f + mat[5] - mat[0] - mat[10]) * 2.0f);
			QuatSet(quat,
					(mat[1] + mat[4]) * s,
					0.5f * s,
					(mat[6] + mat[9]) * s,
					(mat[2] + mat[8]) * s);
			break;
		case 2:
			s = 1 / (sqrt(1.0f + mat[10] - mat[0] - mat[5]) * 2.0f);
			QuatSet(quat,
					(mat[2] + mat[8]) * s,
					(mat[6] + mat[9]) * s,
					0.5f * s,
					(mat[1] + mat[4]) * s);
					
			break;
		};
	}

	QuatNormalize(quat);
}

void Matrix3x4ToQuat( const matrix3x4 mat, quaternion quat ) {
	vec_t trace = mat[0][0] + mat[1][1] + mat[2][2] + 1.0f;
	vec_t s;

	if( trace > 0.0f ) {
		// 'Instant' calculation
		s = 0.5f  / (vec_t)sqrt(trace);
		quat[3] = 0.25 / s;
		quat[0] = (mat[2][1] - mat[1][2]) * s;
		quat[1] = (mat[0][2] - mat[2][0]) * s;
		quat[2] = (mat[1][0] - mat[0][1]) * s;
	} else {
		int n = 0;
		if( mat[1][1] > mat[0][0] )
			n = 1;
		if( mat[2][2] > mat[n][n] )
			n = 2;
		switch(n) {
		case 0:
			s = 1 / (sqrt(1.0f + mat[0][0] - mat[1][1] - mat[2][2]) * 2.0f);
			QuatSet(quat,
					0.5f * s,
					(mat[1][0] + mat[0][1]) * s,
					(mat[2][0] + mat[0][2]) * s,
					(mat[2][1] + mat[1][2]) * s);
			break;
		case 1:
			s = 1 / (sqrt(1.0f + mat[1][1] - mat[0][0] - mat[2][2]) * 2.0f);
			QuatSet(quat,
					(mat[1][0] + mat[0][1]) * s,
					0.5f * s,
					(mat[2][1] + mat[1][2]) * s,
					(mat[2][0] + mat[0][2]) * s);
			break;
		case 2:
			s = 1 / (sqrt(1.0f + mat[2][2] - mat[0][0] - mat[1][1]) * 2.0f);
			QuatSet(quat,
					(mat[2][0] + mat[0][2]) * s,
					(mat[2][1] + mat[1][2]) * s,
					0.5f * s,
					(mat[1][0] + mat[0][1]) * s);
					
			break;
		};
	}

	QuatNormalize(quat);
}


void QuatSlerp( quaternion from, quaternion to, float t, quaternion res ) {
	quaternion	to1;
	vec_t		omega, cosom, sinom, scale0, scale1;

	// calc cosine
	cosom = (vec_t)DotProduct4(from, to);//from->x*to->x + from->y*to->y + from->z*to->z + from->w*to->w;

	// adjust signs (if necessary)
	if( cosom <0.0 ) {
		cosom = -cosom;
		VectorScale4(to, -1.0f, to1);
	} else  {
		VectorCopy4(to, to1);
	}

	// calculate coefficients
	if( (1.0 - cosom) > 0.001 ) {
		// standard case (slerp)
		omega = acos(cosom);
		sinom = 1.0 / sin(omega);
		scale0 = sin((1.0 - t) * omega) * sinom;
		scale1 = sin(t * omega) * sinom;
	} else {        
		// "from" and "to" quaternions are very close 
		//  ... so we can do a linear interpolation
		scale0 = 1.0 - t;
		scale1 = t;
	}

	// calculate final values
	res[0] = scale0 * from[0] + scale1 * to1[0];
	res[1] = scale0 * from[1] + scale1 * to1[1];
	res[2] = scale0 * from[2] + scale1 * to1[2];
	res[3] = scale0 * from[3] + scale1 * to1[3];
}

