
#include "seed_utils.h"
#include <string.h>
#include <stdio.h>
#include <io.h>


//

WorkFile::WorkFile(const string& filename, const string& mode)
{
    this->stream = fopen(filename.c_str(), mode.c_str());
}

WorkFile::~WorkFile()
{
	if( this->stream )
    {
		fclose( this->stream );
		this->stream = NULL;
	}
}

FILE*
WorkFile::Stream()
{
	return this->stream;
}

void
WorkFile::Close()
{
	fclose(this->stream);
	this->stream = NULL;
}

//


WorkBuffer::WorkBuffer()
 : _buffer(NULL)
{
    // empty
}

WorkBuffer::WorkBuffer( WorkFile& workFile )
 : _buffer(NULL)
{
    this->Load(workFile);
}

WorkBuffer::WorkBuffer(const string& filename, const string& mode)
 : _buffer(NULL)
{
	this->Load(filename, mode);
}

WorkBuffer::~WorkBuffer()
{
	this->Release();
}

bool
WorkBuffer::Load(WorkFile& workFile)
{
	if( workFile.Stream() )
    {
		this->Release();

		this->_size = filelength( fileno(workFile.Stream()) );
		if( this->_size <= 0 )
			return false;

		this->_buffer = new byte[ this->_size ];
		if( !this->_buffer )
			return false;

		if( fread(this->_buffer, _size*sizeof(byte), 1, workFile.Stream()) )
			return true;
	}

	this->Release();
	return false;
}


bool
WorkBuffer::Load(const string& filename, const string& mode)
{
	WorkFile workFile(filename, mode);
	return this->Load(workFile);
}


void
WorkBuffer::Release()
{
	if( this->_buffer )
    {
		delete [] (byte*)this->_buffer;
		this->_buffer = NULL;
	}

	this->_size=0;
}


void*
WorkBuffer::GetBuffer()
{
	return this->_buffer;
}


long
WorkBuffer::GetBufferSize()
{
	return this->_size;
}

//


const string
GetExtension(const string& filename)
{
    const int len = filename.length();
    const int pos = filename.find_last_of(".");

    return filename.substr(pos, len-pos);
}


const string
StripExtension(const string& filename)
{
    const int pos = filename.find_last_of(".");

    return filename.substr(0, pos);
}


const string
DefaultExtension(const string& filename, const string& ext)
{
    return StripExtension(filename) + ext;
}
