
#ifndef S_SEED_VDPM_H
#define S_SEED_VDPM_H

#include "seed_types.h"

#include <vector>
using std::vector;

typedef struct VGeom_s {
	vec3_t		point;
} VGeom;


typedef struct AVertex_s	AVertex;
typedef struct AFace_s		AFace;

typedef struct Vertex_s {
	AVertex		*avertex;		// active vertex, NULL if inactive
	Vertex_s	*parent;		// parent vertex, NULL if root
	int			i;				// index of vspliti, -1 if leaf
} Vertex;

typedef struct Face_s {
	AFace		*aface;			// active face, NULL if inactive
} Face;

typedef struct Vsplit_s {
	VGeom		vu_vgeom;		// geometry for child vertex v
	Face		*fn[4];			// required neighbors
	float		radius;			// max extent rv of affected region
	float		dir_error;		// directional error
} Vsplit;

typedef struct ListNode_s ListNode;

struct ListNode_s {
	ListNode_s()
		: next(NULL), prev(NULL)
	{ }

	void append( ListNode * node ) {
		ListNode *n = next;
		// get last node;
		while( n->next ) { n = n->next; }
		n->next = node;
	}

	ListNode_s	*next;
	ListNode_s	*prev;
};

class SRMesh {
public:
	SRMesh()
	{	}

public:
	void vsplit( Vertex * vs );
	void update_vmorphs();
	void adapt_refinement();

protected:
	Vertex		*vertices;
	Face		*faces;

	Vsplit		*vsplits;
	ListNode	active_vertices;
	ListNode	active_faces;
};


typedef struct VertexMorph_s	VertexMorph;

struct AVertex_s {
	ListNode	listnode;
	Vertex		*vertex;
	VGeom		vgeom;
	VertexMorph	*vmorph;
};

struct AFace_s {
	ListNode	listnode;		// list strining active faces
	AVertex		*vertices[3];	// ordered counter-clockwise
	AFace		*neighbors[3];	// neighbors[j] across from vertices[j]
	int			texture_id;
};

struct VertexMorph_s {
	bool		coarsening;		// true if coarsening, false if refining
	short		gtime;			// # of geomorph frames remaining
	VGeom		vg_refined;		// refined geometry
	VGeom		vginc;			// increment per fram during morph
};


#endif
